@@ -50,7 +50,14 @@
 
     [matrix registerForDraggedTypes:[NSArray arrayWithObject:NSColorPboardType]];
     [matrix setDelegate:self];
+#ifdef __APPLE__
+    ColoredSwitchButtonCell *protoCell;
+    protoCell = [[ColoredSwitchButtonCell alloc] init];
+    [matrix setPrototype:protoCell];
+    [protoCell release];
+#else
     [matrix setCellClass:[ColoredSwitchButtonCell class]];
+#endif
 
     [entityTypePopUp removeAllItems];
 
@@ -58,8 +65,8 @@
 
     [[NSNotificationCenter defaultCenter]
         addObserver:self
-           selector:@selector(filterEntityNameNotification:)
-               name:@"PajeFilterEntityNameNotification"
+           selector:@selector(filterEntityValueNotification:)
+               name:@"PajeFilterEntityValueNotification"
              object:nil];
 
 #ifndef GNUSTEP
@@ -239,7 +246,7 @@
 
 - (NSArray *)unfilteredObjectsForEntityType:(PajeEntityType *)entityType
 {
-    return [super allNamesForEntityType:entityType];
+    return [super allValuesForEntityType:entityType];
 }
 
 - (void)unselectAll:(id)sender
@@ -266,9 +273,9 @@
     [super hierarchyChanged];
 }
 
-- (void)filterName:(NSString *)name
-      ofEntityType:(PajeEntityType *)entityType
-              show:(BOOL)flag
+- (void)filterValue:(id)value
+       ofEntityType:(PajeEntityType *)entityType
+               show:(BOOL)flag
 {
     NSMutableSet *filter;
 
@@ -276,18 +283,18 @@
 
     if (filter != nil) {
         if (flag) {
-            [filter removeObject:name];
+            [filter removeObject:value];
         } else {
-            [filter addObject:name];
+            [filter addObject:value];
         }
         [self registerDefaultFilter:filter forEntityType:entityType];
         [super dataChangedForEntityType:entityType];
     }
 }
 
-- (void)filterEntityNameNotification:(NSNotification *)notification
+- (void)filterEntityValueNotification:(NSNotification *)notification
 {
-    NSString *name;
+    id value;
     PajeEntityType *entityType;
     BOOL flag;
     NSDictionary *userInfo;
@@ -297,13 +304,14 @@
         return;
 
     entityType = [userInfo objectForKey:@"EntityType"];
-    name = [userInfo objectForKey:@"EntityName"];
+    value = [userInfo objectForKey:@"EntityValue"];
     flag = [[userInfo objectForKey:@"Show"] isEqualToString:@"YES"];
 
-    if (entityType != nil && name != nil)
-        [self filterName:name
-            ofEntityType:entityType
-                    show:flag];
+    if (entityType != nil && value != nil) {
+        [self filterValue:value
+             ofEntityType:entityType
+                     show:flag];
+    }
     [self synchronizeMatrix];        
 }
 
@@ -313,9 +321,9 @@
     PajeEntityType* entityType = [self selectedEntityType];
 
     if (cell != nil) {
-        [self filterName:[cell representedObject]
-            ofEntityType:entityType
-                    show:[cell state]];
+        [self filterValue:[cell representedObject]
+             ofEntityType:entityType
+                     show:[cell state]];
     }
 }
 
@@ -340,47 +348,21 @@
     filter = [filters objectForKey:entityType];
     names = [[self unfilteredObjectsForEntityType:entityType] objectEnumerator];
 
-    while ([matrix cellAtRow:0 column:0]) {
+    while ([matrix numberOfRows] > 0
+            && ![[matrix cellAtRow:0 column:0]
+                            isKindOfClass:[ColoredSwitchButtonCell class]]) {
         [matrix removeRow:0];
     }
+    [matrix renewRows:[[self unfilteredObjectsForEntityType:entityType] count]
+              columns:1];
 
     while ((entityName = [names nextObject]) != nil) {
-        [matrix addRow];
         cell = [matrix cellAtRow:i column:0];
         [cell setState:!([filter containsObject:entityName])];
         [cell setRepresentedObject:entityName];
-#if 0
-#ifndef xxGNUSTEP
-        NSMutableAttributedString *title;
-        title = [[[NSMutableAttributedString alloc] initWithString:[entityName description]] autorelease];
-#if defined(__APPLE__)// || defined(GNUSTEP)
-        [title replaceCharactersInRange:NSMakeRange(0,0) withString:@"# "];
-        [title addAttribute:NSFontAttributeName
-                      value:[NSFont boldSystemFontOfSize:16]
-                      range:NSMakeRange(0,1)];
-        [title addAttribute:NSFontAttributeName
-                      value:[NSFont systemFontOfSize:12]
-                      range:NSMakeRange(1, [title length]-1)];
-#else
-        // 0x25a0 is unicode for a solid small square
-        [title replaceCharactersInRange:NSMakeRange(0,0)
-               withString:[NSString stringWithCharacter:0x25a0]];
-        [title replaceCharactersInRange:NSMakeRange(1,0) withString:@" "];
-#endif
-        [title addAttribute:NSForegroundColorAttributeName
-                      value:[inputComponent colorForName:entityName
-                                            ofEntityType:entityType]
-                      range:NSMakeRange(0,1)];
-        [title fixAttributesInRange:NSMakeRange(0, [title length])];
-        [cell setAttributedTitle:title];
-#else /*!GNUSTEP*/
         [cell setTitle:[entityName description]];
-#endif
-#else
-        [cell setTitle:[entityName description]];
-        [cell setColor:[inputComponent colorForName:entityName
-                                            ofEntityType:entityType]];
-#endif
+        [cell setColor:[inputComponent colorForValue:entityName
+                                        ofEntityType:entityType]];
         [cell setHighlightsBy:NSChangeBackgroundCellMask];
         i++;
     }
@@ -448,33 +430,64 @@
     }
 }
 
+- (NSEnumerator *)enumeratorOfCompleteEntitiesTyped:(PajeEntityType *)entityType
+                                        inContainer:(PajeContainer *)container
+                                           fromTime:(NSDate *)start
+                                             toTime:(NSDate *)end
+                                        minDuration:(double)minDuration
+{
+    NSEnumerator *origEnum;
+    NSMutableSet *filter;
+    
+    if ([self isHiddenEntityType:entityType]) {
+        NSWarnLog(@"enumerating hidden type %@", entityType);
+        return nil;
+    }
+
+    origEnum = [inputComponent enumeratorOfCompleteEntitiesTyped:entityType
+                                                     inContainer:container
+                                                        fromTime:start
+                                                          toTime:end
+                                                     minDuration:minDuration];
+
+    filter = [filters objectForKey:entityType];
+    if (filter != nil) {
+        return [[[FilteredEnumerator alloc] 
+                        initWithEnumerator:origEnum
+                                    filter:self
+                                  selector:@selector(filterHiddenEntity:filter:)
+                                   context:filter] autorelease];
+    } else {
+        return origEnum;
+    }
+}
+
 - (BOOL)isHiddenEntityType:(PajeEntityType*)entityType
 {
     return [hiddenEntityTypes member:entityType] != nil;
 }
 
-- (BOOL)filterHiddenEntity:(PajeEntity *)entity
-                    filter:(NSSet *)filter
+- (id)filterHiddenEntity:(PajeEntity *)entity
+                  filter:(NSSet *)filter
 {
-    if ([filter containsObject:[self nameForEntity:entity]]) {
+    if ([filter containsObject:[self valueForEntity:entity]]) {
         return nil;
     } else {
         return entity;
     }
 }
 
-- (NSArray *)allNamesForEntityType:(PajeEntityType *)entityType
+- (NSArray *)allValuesForEntityType:(PajeEntityType *)entityType
 {
-    NSArray *allNames = [inputComponent allNamesForEntityType:entityType];
+    NSArray *allValues = [inputComponent allValuesForEntityType:entityType];
     NSMutableSet *filter = [filters objectForKey:entityType];
 
-    if (filter) {
-        NSMutableSet *set = [NSMutableSet setWithArray:allNames];
+    if (filter != nil) {
+        NSMutableSet *set = [NSMutableSet setWithArray:allValues];
         [set minusSet:filter];
-        return [set allObjects];
-    } else {
-        return allNames;
+        allValues = [set allObjects];
     }
+    return allValues;
 }
 
 
@@ -484,7 +497,7 @@
 
 - (void)highlightCell:(id)cell inMatrix:(NSMatrix *)m
 {
-    int r, c;
+    NSInteger r, c;
     static id highlightedCell; // the last cell that has been highlighted
     
     if (highlightedCell && ![highlightedCell isEqual:cell]) {
@@ -544,7 +557,7 @@
         
         dragColor = [NSColor colorFromPasteboard:[sender draggingPasteboard]];
         [inputComponent setColor:dragColor
-                         forName:[cell representedObject]
+                        forValue:[cell representedObject]
                     ofEntityType:entityType];
         return YES;
     }
