@@ -24,13 +24,19 @@
 #include <Foundation/Foundation.h>
 #include "../General/PajeFilter.h"
 
-#define CHUNK_SIZE 1000000
+#define CHUNK_SIZE 1500000
 
-@interface PajeFileReader: PajeComponent <PajeReader>
+@interface FileReader: PajeComponent <PajeReader>
 {
     NSFileHandle *inputFile;
     NSString *inputFilename;
     BOOL hasMoreData;
+
+    NSMutableArray *chunkInfo;
+    unsigned currentChunk;
+
+    unsigned long long sizeOfFile;
+    unsigned long long userChunkSize;
 }
 
 - (id)initWithController:(PajeTraceController *)c;
@@ -40,9 +46,13 @@
 - (void)readNextChunk;
 - (BOOL)hasMoreData;
 
+- (void)startChunk:(int)chunkNumber;
+- (void)endOfChunkLast:(BOOL)last;
+
 - (void)raise:(NSString *)reason;
 
-- (void)encodeCheckPointWithCoder:(NSCoder *)coder;
-- (void)decodeCheckPointWithCoder:(NSCoder *)coder;
+- (NSNumber*) readingPorcentage;
+
+- (void) setUserChunkSize: (unsigned long long)cs;
 @end
 #endif
