@@ -37,19 +37,25 @@
 @interface ChunkArray : NSObject
 {
     PSortedArray *chunks;
+    int firstIndex;
 }
 
 - (id)init;
 
 - (void)dealloc;
 
+// enumerate all entities that end after startTime and start before endTime,
+// in reverse endTime order.
 - (NSEnumerator *)enumeratorOfEntitiesFromTime:(NSDate *)startTime
                                         toTime:(NSDate *)endTime;
+// enumerate all entities that end after startTime and not after endTime,
+// in increasing endTime order.
+- (NSEnumerator *)enumeratorOfCompleteEntitiesFromTime:(NSDate *)startTime
+                                             untilTime:(NSDate *)endTime;
 
 - (void)addChunk:(EntityChunk *)chunk;
 
-- (void)removeChunksBeforeTime:(NSDate *)time;
-- (void)removeChunksAfterTime:(NSDate *)time;
+- (EntityChunk *)chunkAtIndex:(int)index;
+- (void)setFirstIndex:(int)index;
 @end
-
 #endif
