@@ -230,15 +230,16 @@
 
     cell = [containerTypesBrowser selectedCellInColumn:column];
     containerType = [cell representedObject];
-    if (containerType == nil)
+    if (containerType == nil) {
         return;
+    }
     isContainer = ![cell isLeaf];
 
     if (isContainer || containersOnly) {
         containerEnum = [filter enumeratorOfContainersTyped:containerType
                                                 inContainer:upperContainer];
     } else {
-        containerEnum = [[filter allNamesForEntityType:containerType] objectEnumerator];
+        containerEnum = [[filter allValuesForEntityType:containerType] objectEnumerator];
     }
     for (row = 0; (container = [containerEnum nextObject]) != nil; row++) {
         NSBrowserCell *cell;
@@ -282,7 +283,7 @@
         [matrix renewRows:rows columns:1];
         cell = [matrix cellAtRow:rows-1 column:0];
         [cell setRepresentedObject:entityType];
-        [cell setStringValue:[entityType name]];
+        [cell setStringValue:[entityType description]];
         // FIXME: if containersOnly, containers that do not contain containers
         //        should be leaves
         [cell setLeaf:!isContainer];
