@@ -41,27 +41,48 @@
     return self;
 }
 
+- (void)setSelector:(SEL)sel
+{
+    valueSelector = sel;
+    [array sortUsingSelector:sel];
+}
+
 - (void)dealloc
 {
     [array release];
     [super dealloc];
 }
 
-- (unsigned)count
+- (NSUInteger)count
 {
     return [array count];
 }
 
-- (id)objectAtIndex:(unsigned)index
+- (id)objectAtIndex:(NSUInteger)index
 {
     return [array objectAtIndex:index];
 }
 
-- (void)removeObjectAtIndex:(unsigned)index
+- (id)lastObject
+{
+    return [array lastObject];
+}
+
+- (void)removeLastObject
+{
+    [array removeLastObject];
+}
+
+- (void)removeObjectAtIndex:(NSUInteger)index
 {
     [array removeObjectAtIndex:index];
 }
 
+- (void) removeObjectsInRange: (NSRange)aRange
+{
+    [array removeObjectsInRange:aRange];
+}
+
 - (void)removeObject:(id)obj
 {
     id<Comparing> value = [obj performSelector:valueSelector];
@@ -102,6 +123,43 @@
     [array removeAllObjects];
 }
 
+- (void)addObject:(id)obj 
+        withValue:(id<Comparing>)objValue 
+             left:(int)left
+            right:(int)right
+            pivot:(int)pivot
+{
+  int size = right-left;
+  if (size == 0){
+    [array insertObject:obj atIndex:pivot];
+    return;
+  }
+
+  id<Comparing> pivotValue = [[array objectAtIndex:pivot]
+                              performSelector: valueSelector];
+  NSComparisonResult comp = [pivotValue compare:objValue];
+
+  //check if size is 1
+  if (size == 1){
+    if (comp == NSOrderedDescending){
+      [array insertObject:obj atIndex:pivot];
+    }else{
+      [array insertObject:obj atIndex:pivot+1];
+    }
+    return;
+  }
+
+  //recurse normally
+  if (comp == NSOrderedAscending){
+    [self addObject:obj withValue:objValue
+               left:pivot right:right pivot:pivot+(right-pivot)/2];
+  }else if(comp == NSOrderedDescending){
+    [self addObject:obj withValue:objValue
+               left:left right:pivot pivot:left+(pivot-left)/2];
+  }else{
+    [array insertObject:obj atIndex:pivot+1];
+  }
+}
 
 - (void)addObject:(id)obj
 {
@@ -115,13 +173,9 @@
         [array addObject:obj];
     } else {
         // find the place to insert
-        pos-=2;    // no need to retest the last one.
-        while (pos >= 0
-               && [value compare:[[array objectAtIndex:pos]
-                  performSelector:valueSelector]] == NSOrderedAscending)
-                  // obj < array[pos]
-            pos--;
-        [array insertObject:obj atIndex:++pos];
+        int left = 0, right = [array count];
+        int pivot = [array count]/2;
+        [self addObject:obj withValue:value left:left right:right pivot:pivot];
     }
 }
 
@@ -178,7 +232,7 @@ int delta_d_tctr;
 double delta_d_time;
 double delta_d_time0;
 NSDate *delta_d_t0;
-- (unsigned)indexOfFirstObjectNotBeforeValue:(id<Comparing>)value
+- (NSUInteger)indexOfFirstObjectNotBeforeValue:(id<Comparing>)value
 {
     int lo, hi, pivot, cnt;
     lo = 0;
@@ -210,24 +264,56 @@ NSDate *delta_d_t0;
         return lo;
 }
 
-- (unsigned)indexOfLastObjectNotAfterValue:(id<Comparing>)value
+- (NSUInteger)indexOfFirstObjectAfterValue:(id<Comparing>)value
 {
+    if (value == nil) {
+        return 0;
+    }
     unsigned index = [self indexOfFirstObjectNotBeforeValue:value];
 
-    if (index < [array count]) {
+    while (index < [array count]) {
         id arrValue;
         arrValue = [[array objectAtIndex:index] performSelector:valueSelector];
-        if ([value compare:arrValue] == NSOrderedSame) {
-            return index;
+        if ([value compare:arrValue] == NSOrderedAscending) { // v<a[i]
+            break;
         }
+        index++;
     }
-    if (index > 0) {
-        return index - 1;
-    }
-    return NSNotFound;
+    return index;
 }
 
-- (unsigned)indexOfObjectWithValue:(id<Comparing>)value
+- (NSUInteger)indexOfLastObjectBeforeValue:(id<Comparing>)value;
+{
+    if (value == nil) {
+        return 0;
+    }
+    int index = [self indexOfFirstObjectNotBeforeValue:value];
+
+    while (index >= 0) {
+        id arrValue;
+        if (index == 0) {
+            return NSNotFound;
+        }
+        index --;
+        arrValue = [[array objectAtIndex:index] performSelector:valueSelector];
+        if ([value compare:arrValue] == NSOrderedDescending) { // v>a[i]
+            break;
+        }
+    }
+    return index;
+}
+
+- (NSUInteger)indexOfLastObjectNotAfterValue:(id<Comparing>)value
+{
+    int index = [self indexOfFirstObjectAfterValue:value];
+
+    if (index == 0) {
+        return NSNotFound;
+    }
+    return index - 1;
+}
+
+- (NSUInteger)indexOfObjectWithValue:(id<Comparing>)value
 {
     unsigned index = [self indexOfFirstObjectNotBeforeValue:value];
 
@@ -241,11 +327,16 @@ NSDate *delta_d_t0;
     return NSNotFound;
 }
 
-- (unsigned)indexOfObject:(id)anObject
+- (NSUInteger)indexOfObject:(id)anObject
 {
     return [self indexOfObjectWithValue:[anObject performSelector:valueSelector]];
 }
 
+- (NSEnumerator *)reverseObjectEnumeratorWithRange:(NSRange)range
+{
+    return [array reverseObjectEnumeratorWithRange:range];
+}
+
 - (NSEnumerator *)objectEnumerator
 {
     return [array objectEnumerator];
@@ -258,7 +349,27 @@ NSDate *delta_d_t0;
 
 - (NSEnumerator *)objectEnumeratorAfterValue:(id<Comparing>)value
 {
-    int firstIndex;
+    unsigned firstIndex;
+    NSRange range;
+
+    firstIndex = [self indexOfFirstObjectAfterValue:value];
+    range = NSMakeRange(firstIndex, [array count] - firstIndex);
+    return [array objectEnumeratorWithRange:range];
+}
+
+- (NSEnumerator *)objectEnumeratorNotAfterValue:(id<Comparing>)value
+{
+    unsigned lastIndex;
+    NSRange range;
+
+    lastIndex = [self indexOfFirstObjectAfterValue:value];
+    range = NSMakeRange(0, lastIndex);
+    return [array objectEnumeratorWithRange:range];
+}
+
+- (NSEnumerator *)objectEnumeratorNotBeforeValue:(id<Comparing>)value
+{
+    unsigned firstIndex;
     NSRange range;
 
     firstIndex = [self indexOfFirstObjectNotBeforeValue:value];
@@ -266,9 +377,51 @@ NSDate *delta_d_t0;
     return [array objectEnumeratorWithRange:range];
 }
 
+- (NSEnumerator *)objectEnumeratorNotBeforeValue:(id<Comparing>)value1
+                                   notAfterValue:(id<Comparing>)value2
+{
+    NSUInteger firstIndex;
+    NSUInteger lastIndex;
+    NSRange range;
+
+    lastIndex = [self indexOfLastObjectNotAfterValue:value2];
+    if (lastIndex == NSNotFound) {
+        return nil;
+    }
+    firstIndex = [self indexOfFirstObjectNotBeforeValue:value1];
+    range = NSMakeRange(firstIndex, lastIndex - firstIndex + 1);
+    return [array objectEnumeratorWithRange:range];
+}
+
+- (NSEnumerator *)objectEnumeratorAfterValue:(id<Comparing>)value1
+                               notAfterValue:(id<Comparing>)value2
+{
+    NSUInteger firstIndex;
+    NSUInteger lastIndex;
+    NSRange range;
+
+    lastIndex = [self indexOfLastObjectNotAfterValue:value2];
+    if (lastIndex == NSNotFound) {
+        return nil;
+    }
+    firstIndex = [self indexOfFirstObjectAfterValue:value1];
+    range = NSMakeRange(firstIndex, lastIndex - firstIndex + 1);
+    return [array objectEnumeratorWithRange:range];
+}
+
 - (NSEnumerator *)reverseObjectEnumeratorAfterValue:(id<Comparing>)value
 {
-    int firstIndex;
+    unsigned firstIndex;
+    NSRange range;
+
+    firstIndex = [self indexOfFirstObjectAfterValue:value];
+    range = NSMakeRange(firstIndex, [array count] - firstIndex);
+    return [array reverseObjectEnumeratorWithRange:range];
+}
+
+- (NSEnumerator *)reverseObjectEnumeratorNotBeforeValue:(id<Comparing>)value
+{
+    unsigned firstIndex;
     NSRange range;
 
     firstIndex = [self indexOfFirstObjectNotBeforeValue:value];
@@ -276,12 +429,6 @@ NSDate *delta_d_t0;
     return [array reverseObjectEnumeratorWithRange:range];
 }
 
-
-- (void) removeObjectsInRange: (NSRange)aRange
-{
-    [array removeObjectsInRange:aRange];
-}
-
 // NSCoding Protocol
 - (void)encodeWithCoder:(NSCoder *)coder
 {
