@@ -93,6 +93,22 @@
     return nil;
 }
 
+- (NSEnumerator *)enumeratorOfEntitiesTyped:(PajeEntityType *)type
+                                   fromTime:(NSDate *)start
+                                     toTime:(NSDate *)end
+{
+    [self _subclassResponsibility:_cmd];
+    return nil;
+}
+
+- (NSEnumerator *)enumeratorOfCompleteEntitiesTyped:(PajeEntityType *)type
+                                           fromTime:(NSDate *)start
+                                             toTime:(NSDate *)end
+{
+    [self _subclassResponsibility:_cmd];
+    return nil;
+}
+
 // NSCoding protocol
 - (void)encodeWithCoder:(NSCoder *)coder
 {
@@ -106,4 +122,21 @@
     Assign(subContainers, [coder decodeObject]);
     return self;
 }
+
+- (double)minValueForEntityType:(PajeEntityType *)type
+{
+    return 0.0;
+}
+
+- (double)maxValueForEntityType:(PajeEntityType *)type
+{
+    return 0.0;
+}
+
+
+- (void)verifyMinMaxOfEntityType:(PajeEntityType *)type
+                       withValue:(double)value
+{
+  return;
+}
 @end
