@@ -226,21 +226,21 @@ static NSMutableArray *allInstances;
 - (IBAction)filterEntityName:(id)sender
 {
     PajeEntityType *entityType;
-    id entityName;
+    id entityValue;
 
     entityType = [inspectedEntity entityType];
-    entityName = [inspectedEntity name];
-    if (entityType != nil && entityName != nil) {
+    entityValue = [inspectedEntity value];
+    if (entityType != nil && entityValue != nil) {
         NSString *filtering;
         NSDictionary *userInfo;
         filtering = [sender state] ? @"YES" : @"NO";
         userInfo = [NSDictionary dictionaryWithObjectsAndKeys:
                                          entityType, @"EntityType",
-                                         entityName, @"EntityName",
+                                         entityValue, @"EntityValue",
                                          filtering, @"Show",
                                          nil];
         [[NSNotificationCenter defaultCenter]
-                postNotificationName:@"PajeFilterEntityNameNotification"
+                postNotificationName:@"PajeFilterEntityValueNotification"
                               object:self
                             userInfo:userInfo];
     }
@@ -252,9 +252,12 @@ static NSMutableArray *allInstances;
 {
     NSBox *box;
     int index;
-    float curY = 0;
+    float curY;
     float titleWidth = 0;
+    float leftMargin;
+    float deltaHeight;
     NSTextField *field1;
+    NSRect boxFrame;
 
     NSAssert([titles count] == [values count],
              @"count of fields and values do not match");
@@ -262,6 +265,12 @@ static NSMutableArray *allInstances;
     // I don't know a better way of copying a box...
     box = [NSUnarchiver unarchiveObjectWithData:archivedBox];
     field1 = [NSUnarchiver unarchiveObjectWithData:archivedTitleField];
+    leftMargin = NSMinX([field1 frame]);
+    curY = NSMinY([field1 frame]);
+#ifdef __APPLE__
+    curY -= 4;
+#endif
+    deltaHeight = NSHeight([box frame]) - NSMaxY([field1 frame]);
 
     [box setTitle:boxTitle];
 
@@ -284,6 +293,9 @@ static NSMutableArray *allInstances;
         
         [tField setStringValue:title];
         [vField setStringValue:value];
+        height = MAX([[tField cell] cellSize].height,
+                     [[vField cell] cellSize].height);
+        
         if ([value isEqual:@"**AQUI**"]) {
             NSCell *cell = [[InnerStatesCell alloc] init];
             [cell setRepresentedObject:inspectedEntity];
@@ -291,17 +303,18 @@ static NSMutableArray *allInstances;
             [cell release];
         }
         
-        height = MAX([[tField cell] cellSize].height,
-                     [[vField cell] cellSize].height);
-        
-        [tField setFrame:NSMakeRect(0, curY, titleWidth, height)];
-        [vField setFrame:NSMakeRect(titleWidth + 2, curY, 10, height)];
+        [tField setFrame:NSMakeRect(leftMargin, curY, titleWidth, height)];
+        [vField setFrame:NSMakeRect(leftMargin + titleWidth + 2, curY, 10, height)];
         [box addSubview:tField];
         [box addSubview:vField];
         
         curY += height+2;
     }
-    [box sizeToFit];
+    boxFrame = [box frame];
+    boxFrame.size.height = curY - 2 + deltaHeight;
+    boxFrame.size.width = leftMargin + titleWidth + 2 + 10 + leftMargin;
+    [box setFrame:boxFrame];
+    //[box sizeToFit];
 
     return box;
 }
@@ -391,10 +404,10 @@ static NSMutableArray *allInstances;
     fieldValues = [NSMutableArray array];
     fieldObjects = [NSMutableArray array];
     while ([container container] != nil && ![container isEqual:upto]) {
-        [fieldTitles insertObject:[[container entityType] name] atIndex:0];
-        [fieldValues insertObject:[container name] atIndex:0];
-        [fieldObjects insertObject:container atIndex:0];
-        container = [container container];
+      [fieldTitles insertObject:[[container entityType] description] atIndex:0];
+      [fieldValues insertObject:[container name] atIndex:0];
+      [fieldObjects insertObject:container atIndex:0];
+      container = [container container];
     }
 
     box = [self boxWithTitle:title
@@ -516,7 +529,7 @@ static NSMutableArray *allInstances;
         if (subCount <= 5) count = subCount;
         else count = 4;
         for (i = 0; i < count; i++) {
-            [fieldTitles addObject:[inspectedEntity subNameAtIndex:i]];
+            [fieldTitles addObject:[inspectedEntity subValueAtIndex:i]];
             if ([inspectedEntity drawingType] == PajeStateDrawingType) {
                 value = [inspectedEntity subDurationAtIndex:i];
                 [fieldValues addObject:
@@ -559,7 +572,8 @@ static NSMutableArray *allInstances;
 - (void)addGlobalFields
 {
     // global fields
-    [nameField setStringValue:[filter nameForEntity:inspectedEntity]];
+    [nameField setStringValue:[[filter valueForEntity:inspectedEntity] stringValue]];
+    [colorField setTarget:nil];
     [colorField setColor:[filter colorForEntity:inspectedEntity]];
     [colorField setAction:@selector(colorChanged:)];
     [colorField setTarget:self];
@@ -567,7 +581,7 @@ static NSMutableArray *allInstances;
                                  [filter entityTypeForEntity:inspectedEntity]]];
 
     [nonDisplayedFields removeObject:@"PajeEventName"];
-    [nonDisplayedFields removeObject:@"Name"];
+    //[nonDisplayedFields removeObject:@"Value"];
     [nonDisplayedFields removeObject:@"Color"];
 
     [nonDisplayedFields removeObject:@"EntityType"];
@@ -617,11 +631,15 @@ static NSMutableArray *allInstances;
 - (void)addScriptBox
 {
     NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
+    NSString *scriptPath;
+    NSString *scriptFieldName;
 
-    [scriptPathField setStringValue:
-            [defaults objectForKey:@"EntityInspectorScriptPath"]];
-    [scriptFieldNameField setStringValue:
-            [defaults objectForKey:@"EntityInspectorScriptFieldName"]];
+    scriptPath = [defaults objectForKey:@"EntityInspectorScriptPath"];
+    scriptFieldName = [defaults objectForKey:@"EntityInspectorScriptFieldName"];
+    if (scriptPath == nil) scriptPath = @"";
+    if (scriptFieldName == nil) scriptFieldName = @"";
+    [scriptPathField setStringValue:scriptPath];
+    [scriptFieldNameField setStringValue:scriptFieldName];
 
     if ([[filter relatedEntitiesForEntity:inspectedEntity] count] == 0) {
         [scriptFieldSourceMatrix selectCellWithTag:0];
@@ -648,7 +666,7 @@ static NSMutableArray *allInstances;
         while ((related = [relatedEnum nextObject]) != nil) {
             NSButtonCell *cell;
             cell = [relatedEntitiesMatrix cellAtRow:row++ column:0];
-            [cell setTitle:[filter nameForEntity:related]];
+            [cell setTitle:[[filter valueForEntity:related] stringValue]];
             [cell setRepresentedObject:related];
             [cell setTarget:self];
             [cell setAction:@selector(entityClicked:)];
