@@ -19,6 +19,7 @@
 */
 //#define COMPATIBILITY
 #include "PajeFilter.h"
+#include "Macros.h"
 #include "../Paje/PajeTraceController.h"
 
 
@@ -49,17 +50,27 @@
 
 - (void)dealloc
 {
-    [outputComponent release]; // HACK (see setOutputComponent:)
+    [self disconnectComponent];
     [super dealloc];
 }
 
-- (void)setInputComponent:(PajeComponent *)component
+- (void)disconnectComponent
 {
-    inputComponent = (id)component;
+    [self setInputComponent:nil];
+    [self setOutputComponent:nil];
 }
 
-- (void)setOutputComponent:(PajeComponent *)component
+- (void)setInputComponent:(PajeFilter *)component
 {
+    inputComponent = component;
+}
+
+- (void)setOutputComponent:(PajeFilter *)component
+{
+    if (component == nil) {
+        Assign(outputComponent, component);
+        return;
+    }
     if (outputComponent != nil) { // HACK
         if ([outputComponent isKindOfClass: [NSMutableArray class]]) {
             [(NSMutableArray *)outputComponent addObject:component];
@@ -68,7 +79,7 @@
             outputComponent = [[NSMutableArray arrayWithObjects:outputComponent, component, nil] retain];
         }
     } else {
-        outputComponent = [component retain];
+      outputComponent = [component retain];
     }
 }
 
@@ -85,6 +96,11 @@
         [outputComponent performSelector:outputSelector withObject:entity];
 }
 
+- (BOOL)canEndChunkBefore:(id)entity
+{
+    return [outputComponent canEndChunkBefore:entity];
+}
+
 /* only for use by simulator */
 - (NSString *)traceDescription
 {
@@ -112,7 +128,7 @@
     return nil;
 }
 
-- (NSString *)filterKeyEquivalent;
+- (NSString *)filterKeyEquivalent
 {
     return @"";
 }
@@ -126,6 +142,30 @@
 {
     [controller registerTool:filter];
 }
+
+- (void)startChunk:(int)chunkNumber
+{
+    if ([outputComponent isKindOfClass:[NSArray class]]) {// HACK
+        [NSException raise:@"Unimplemented"
+                    format:@"[-startChunk:]"
+                            " not implemented for group of components\n"
+                            "Simplify your component graph"];
+    } else {
+        [outputComponent startChunk:chunkNumber];
+    }
+}
+
+- (void)endOfChunkLast:(BOOL)last
+{
+    if ([outputComponent isKindOfClass:[NSArray class]]) {// HACK
+        [NSException raise:@"Unimplemented"
+                    format:@"[-endOfChunkLast:]"
+                            " not implemented for group of components\n"
+                            "Simplify your component graph"];
+    } else {
+        [outputComponent endOfChunkLast:last];
+    }
+}
 @end
 
 
@@ -135,6 +175,15 @@
 //
 // Deal with notifications
 //
+- (void)timeLimitsChanged
+{
+    if ([outputComponent isKindOfClass:[NSArray class]]) {
+        [(NSArray*)outputComponent makeObjectsPerformSelector:_cmd];
+    } else {
+        [outputComponent timeLimitsChanged];
+    }
+}
+
 - (void)dataChangedForEntityType:(PajeEntityType *)entityType
 {
     if ([outputComponent isKindOfClass:[NSArray class]]) {
@@ -145,6 +194,16 @@
     }
 }
 
+- (void)limitsChangedForEntityType:(PajeEntityType *)entityType
+{
+    if ([outputComponent isKindOfClass:[NSArray class]]) {
+	[(NSArray *)outputComponent makeObjectsPerformSelector:_cmd 
+                                                    withObject:entityType];
+    } else {
+        [outputComponent limitsChangedForEntityType:entityType];
+    }
+}
+
 - (void)colorChangedForEntityType:(PajeEntityType *)entityType
 {
     if ([outputComponent isKindOfClass:[NSArray class]]) {
@@ -238,11 +297,11 @@ ofContainersTyped:(PajeEntityType *)containerType
 }
 
 - (void)setColor:(NSColor *)color
-         forName:(NSString *)name
+        forValue:(id)value
     ofEntityType:(PajeEntityType *)entityType
 {
     [inputComponent setColor:color
-                     forName:name
+                    forValue:value
                 ofEntityType:entityType];
 }
 
@@ -253,16 +312,19 @@ ofContainersTyped:(PajeEntityType *)containerType
                forEntityType:entityType];
 }
 
-- (void)setColor:(NSColor *)color forEntity:(id<PajeEntity>)entity;
+- (void)setColor:(NSColor *)color
+       forEntity:(id<PajeEntity>)entity;
 {
     [inputComponent setColor:color
                    forEntity:entity];
 }
 
 
-- (void)verifyStartTime:(NSDate *)start endTime:(NSDate *)end
+- (void)verifyStartTime:(NSDate *)start
+                endTime:(NSDate *)end
 {
-    [inputComponent verifyStartTime:start endTime:end];
+    [inputComponent verifyStartTime:start
+                            endTime:end];
 }
 
 
@@ -319,7 +381,7 @@ ofContainersTyped:(PajeEntityType *)containerType
     return [inputComponent containedTypesForContainerType:containerType];
 }
 
-- (PajeEntityType *)containerTypeForType:(PajeEntityType *)entityType
+- (PajeContainerType *)containerTypeForType:(PajeEntityType *)entityType
 {
     return [inputComponent containerTypeForType:entityType];
 }
@@ -337,6 +399,19 @@ ofContainersTyped:(PajeEntityType *)containerType
                                          minDuration:minDuration];
 }
 
+- (NSEnumerator *)enumeratorOfCompleteEntitiesTyped:(PajeEntityType *)entityType
+                                        inContainer:(PajeContainer *)container
+                                           fromTime:(NSDate *)start
+                                             toTime:(NSDate *)end
+                                        minDuration:(double)minDuration
+{
+    return [inputComponent enumeratorOfCompleteEntitiesTyped:entityType
+                                                 inContainer:container
+                                                    fromTime:start
+                                                      toTime:end
+                                                 minDuration:minDuration];
+}
+
 - (NSEnumerator *)enumeratorOfContainersTyped:(PajeEntityType *)entityType
                                   inContainer:(PajeContainer *)container
 {
@@ -344,9 +419,9 @@ ofContainersTyped:(PajeEntityType *)containerType
                                            inContainer:container];
 }
 
-- (NSArray *)allNamesForEntityType:(PajeEntityType *)entityType
+- (NSArray *)allValuesForEntityType:(PajeEntityType *)entityType
 {
-    return [inputComponent allNamesForEntityType:entityType];
+    return [inputComponent allValuesForEntityType:entityType];
 }
 
 - (NSString *)descriptionForEntityType:(PajeEntityType *)entityType
@@ -354,25 +429,25 @@ ofContainersTyped:(PajeEntityType *)containerType
     return [inputComponent descriptionForEntityType:entityType];
 }
 
-- (NSNumber *)minValueForEntityType:(PajeEntityType *)entityType
+- (double)minValueForEntityType:(PajeEntityType *)entityType
 {
     return [inputComponent minValueForEntityType:entityType];
 }
 
-- (NSNumber *)maxValueForEntityType:(PajeEntityType *)entityType
+- (double)maxValueForEntityType:(PajeEntityType *)entityType
 {
     return [inputComponent maxValueForEntityType:entityType];
 }
 
-- (NSNumber *)minValueForEntityType:(PajeEntityType *)entityType
-                        inContainer:(PajeContainer *)container
+- (double)minValueForEntityType:(PajeEntityType *)entityType
+                    inContainer:(PajeContainer *)container
 {
     return [inputComponent minValueForEntityType:entityType
                                      inContainer:container];
 }
 
-- (NSNumber *)maxValueForEntityType:(PajeEntityType *)entityType
-                        inContainer:(PajeContainer *)container
+- (double)maxValueForEntityType:(PajeEntityType *)entityType
+                    inContainer:(PajeContainer *)container
 {
     return [inputComponent maxValueForEntityType:entityType
                                      inContainer:container];
@@ -408,9 +483,11 @@ ofContainersTyped:(PajeEntityType *)containerType
                                forEntityType:entityType];
 }
 
-- (NSColor *)colorForName:(NSString *)name ofEntityType:(PajeEntityType *)entityType
+- (NSColor *)colorForValue:(id)value
+              ofEntityType:(PajeEntityType *)entityType
 {
-    return [inputComponent colorForName:name ofEntityType:entityType];
+    return [inputComponent colorForValue:value
+                            ofEntityType:entityType];
 }
 
 - (NSColor *)colorForEntityType:(PajeEntityType *)entityType
@@ -426,9 +503,11 @@ ofContainersTyped:(PajeEntityType *)containerType
     return [inputComponent fieldNamesForEntity:entity];
 }
 
-- (id)valueOfFieldNamed:(NSString *)fieldName forEntity:(id<PajeEntity>)entity
+- (id)valueOfFieldNamed:(NSString *)fieldName
+              forEntity:(id<PajeEntity>)entity
 {
-    return [inputComponent valueOfFieldNamed:fieldName forEntity:entity];
+    return [inputComponent valueOfFieldNamed:fieldName
+                                   forEntity:entity];
 }
 
 
@@ -497,16 +576,26 @@ ofContainersTyped:(PajeEntityType *)containerType
     return [inputComponent drawingTypeForEntity:entity];
 }
 
-- (NSString *)nameForEntity:(id<PajeEntity>)entity
-{
-    return [inputComponent nameForEntity:entity];
-}
-
-- (NSNumber *)valueForEntity:(id<PajeEntity>)entity // for variables
+- (id)valueForEntity:(id<PajeEntity>)entity
 {
     return [inputComponent valueForEntity:entity];
 }
 
+- (double)doubleValueForEntity:(id<PajeEntity>)entity // for variables
+{
+    return [inputComponent doubleValueForEntity:entity];
+}
+
+- (double)minValueForEntity:(id<PajeEntity>)entity // for variables
+{
+    return [inputComponent minValueForEntity:entity];
+}
+
+- (double)maxValueForEntity:(id<PajeEntity>)entity // for variables
+{
+    return [inputComponent maxValueForEntity:entity];
+}
+
 - (NSString *)descriptionForEntity:(id<PajeEntity>)entity
 {
     return [inputComponent descriptionForEntity:entity];
@@ -528,24 +617,32 @@ ofContainersTyped:(PajeEntityType *)containerType
     return [inputComponent subCountForEntity:entity];
 }
 
-- (NSColor *)subColorAtIndex:(unsigned)index forEntity:(id<PajeEntity>)entity
+- (NSColor *)subColorAtIndex:(unsigned)index
+                   forEntity:(id<PajeEntity>)entity
 {
-    return [inputComponent subColorAtIndex:index forEntity:entity];
+    return [inputComponent subColorAtIndex:index
+                                 forEntity:entity];
 }
 
-- (NSString *)subNameAtIndex:(unsigned)index forEntity:(id<PajeEntity>)entity
+- (id)subValueAtIndex:(unsigned)index
+            forEntity:(id<PajeEntity>)entity
 {
-    return [inputComponent subNameAtIndex:index forEntity:entity];
+    return [inputComponent subValueAtIndex:index
+                                 forEntity:entity];
 }
 
-- (double)subDurationAtIndex:(unsigned)index forEntity:(id<PajeEntity>)entity
+- (double)subDurationAtIndex:(unsigned)index
+                   forEntity:(id<PajeEntity>)entity
 {
-    return [inputComponent subDurationAtIndex:index forEntity:entity];
+    return [inputComponent subDurationAtIndex:index
+                                    forEntity:entity];
 }
 
-- (unsigned)subCountAtIndex:(unsigned)index forEntity:(id<PajeEntity>)entity
+- (unsigned)subCountAtIndex:(unsigned)index
+                  forEntity:(id<PajeEntity>)entity
 {
-    return [inputComponent subCountAtIndex:index forEntity:entity];
+    return [inputComponent subCountAtIndex:index
+                                 forEntity:entity];
 }
 
 
@@ -627,11 +724,15 @@ ofContainersTyped:(PajeEntityType *)containerType
     containerEnum = [self enumeratorOfContainersTyped:t
                                           inContainer:[self rootInstance]];
     while ((container = [containerEnum nextObject]) != nil) {
-        if ([n isEqual:[self nameForEntity:container]]) {
+        if ([n isEqual:[self nameForContainer:container]]) {
             break;
         }
     }
     return container;
 }
 
+- (NSString *)nameForContainer:(PajeContainer *)container
+{
+    return [container name];
+}
 @end
