@@ -68,7 +68,9 @@
 {
     [containers release];
     [hierarchyBrowser release];
+#ifndef __APPLE__
     [view release];
+#endif
     [super dealloc];
 }
 
@@ -212,7 +214,7 @@
     NSMutableArray *containerArray;
     NSArray *selectedContainers;
     PajeContainer *firstSelectedContainer;
-    unsigned index, maxIndex;
+    NSUInteger index, maxIndex;
 
     selectedContainers = [[self selectedContainers] allObjects];
     if ((selectedContainers == nil) || ([selectedContainers count] == 0)) {
@@ -241,9 +243,6 @@
                     format:@"Container Not Found: %@", firstSelectedContainer];
     }
     index += moveAmount;
-    if (index < 0) {
-        index = 0;
-    }
     maxIndex = [containerArray count] - [selectedContainers count];
     if (index > maxIndex) {
         index = maxIndex;
