@@ -45,8 +45,8 @@
 - (id)container;
 - (id)entityType;
 
-- (BOOL)isEqual:(OrderKey *)other;
-- (unsigned)hash;
+- (BOOL)isEqual:(id)other;
+- (NSUInteger)hash;
 @end
 
 #endif
