@@ -17,19 +17,18 @@
     along with Paj; if not, write to the Free Software Foundation, Inc.,
     59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 */
+
 #include "PajeController.h"
 #include "PajeTraceController.h"
 #include "PajeCheckPoint.h"
-#if 0
-#ifndef GNUSTEP
-#include "OEObject.h"
-#endif
-#endif
 #include "../General/Protocols.h"
 #include "../General/Macros.h"
 #include "../General/PajeFilter.h"
 #include "../StorageController/Encapsulate.h"
 
+#include <locale.h>
+
+
 @interface NSMenu (RemoveAllItems)
 - (void)removeAllItems;
 @end
@@ -56,6 +55,17 @@ static PajeController *uniqueController;
     return uniqueController;
 }
 
+- (void)awakeFromNib
+{
+    // Couldn't connect to menu in Gorm, only to menuitem...
+    if ([[filtersMenu class] isEqual:[NSMenuItem class]]) {
+        filtersMenu = [(NSMenuItem *)filtersMenu submenu];
+    }
+    if ([[toolsMenu class] isEqual:[NSMenuItem class]]) {
+        toolsMenu = [(NSMenuItem *)toolsMenu submenu];
+    }
+}
+
 - (id)init
 {
     if (uniqueController != nil) {
@@ -63,8 +73,12 @@ static PajeController *uniqueController;
     } else {
         // initialisation code here
         Assign(traceControllers, [NSMutableArray array]);
+        Assign(bundles, [NSMutableDictionary dictionary]);
         
         uniqueController = self;
+
+        // FIXME: find a cleaner way of reading numbers independently of locale
+        setlocale(LC_NUMERIC, "C");
     }
     return uniqueController;
 }
@@ -92,7 +106,9 @@ static PajeController *uniqueController;
     bundlePaths = [[NSUserDefaults standardUserDefaults]
                                        arrayForKey:@"BundlePaths"];
     if (!bundlePaths) {
-        bundlePaths = NSStandardLibraryPaths();
+        bundlePaths = NSSearchPathForDirectoriesInDomains(
+                                            NSAllLibrariesDirectory,
+                                            NSAllDomainsMask, YES);
     }
 
     // insert application path (eases development)
@@ -104,6 +120,7 @@ static PajeController *uniqueController;
         bundlePath = [path stringByAppendingPathComponent:bundleName];
         bundle = [NSBundle bundleWithPath:bundlePath];
         if ([bundle load]) {
+            [bundles setObject:bundle forKey:name];
             return bundle;
         }
     }
@@ -111,8 +128,22 @@ static PajeController *uniqueController;
     return nil;
 }
 
+- (NSBundle *)bundleWithName:(NSString *)name
+{
+    NSBundle *bundle;
+
+    bundle = [bundles objectForKey:name];
+    if (bundle == nil) {
+        [self loadBundleNamed:name];
+        bundle = [bundles objectForKey:name];
+    }
+
+    return bundle;
+}
+
 - (void)loadAllBundles
 {
+return;
     [self loadBundleNamed:@"General"];
     [self loadBundleNamed:@"FileReader"];
     [self loadBundleNamed:@"PajeEventDecoder"];
@@ -201,6 +232,7 @@ static PajeController *uniqueController;
     if (traceController == currentTraceController) {
         [self setCurrentTraceController:nil];
     }
+    [traceController close];
     [traceControllers removeObjectIdenticalTo:traceController];
 }
 
@@ -304,10 +336,10 @@ static PajeController *uniqueController;
     
     [filtersMenu addItemWithTitle:@"Save Configuration"
                            action:@selector(saveFilterConfiguration:)
-                    keyEquivalent:nil];
+                    keyEquivalent:@""];
     [filtersMenu addItemWithTitle:@"Load Configuration"
                            action:@selector(loadFilterConfiguration:)
-                    keyEquivalent:nil];
+                    keyEquivalent:@""];
 
     [filtersMenu sizeToFit];
     [filtersPopUp selectItemWithTitle:previouslySelectedFilterName];
