@@ -34,15 +34,15 @@
     id <PajeReader> reader;
     id <PajeSimulator> simulator;
     id encapsulator;
-    NSMutableArray *components;
+    NSMutableDictionary *components;
     
     NSMutableDictionary *filters;
     NSMutableArray *tools;
 
-    NSString *checkPointDirectory;
-    PSortedArray *checkPoints;
-    
     NSString *configurationName;
+
+    PSortedArray *chunkDates;
+    BOOL timeLimitsChanged;
 }
 
 - (void)registerFilter:(PajeFilter *)filter;
@@ -51,15 +51,12 @@
 - (NSDictionary *)filters;
 - (NSArray *)tools;
 
-- (void)readChunk:(id)sender;
+- (void)readNextChunk:(id)sender;
+- (void)readChunk:(int)chunkNumber;
+- (void)startChunk:(int)chunkNumber;
+- (void)endOfChunkLast:(BOOL)last;
 
-- (void)createCheckPointDirectory;
-- (void)destroyCheckPointDirectory;
-- (NSString *)checkPointDirectory;
-
-- (void)writeCheckPoint;
-- (void)gotoCheckPoint:(PajeCheckPoint *)checkPoint;
-- (void)traceFault:(NSNotification *)notification;
+- (void)missingChunk:(int)chunkNumber;
 
 - (BOOL)openFile:(NSString *)filename;
 
@@ -72,6 +69,9 @@
 
 // Remove a component from the chain.
 - (void)removeComponent:component;
+
+// Close the controller (remove all components)
+- (void)close;
 @end
 
 #endif
