@@ -33,24 +33,19 @@
 #include "../General/PajeFilter.h"
 #include "../General/PajeEvent.h"
 
-#include "SimulContainer.h"
+@class SimulContainer;
 
-#define MAP
 @interface PajeSimul : PajeComponent <PajeSimulator>
 {
     SimulContainer *rootContainer;
-#ifdef MAP
-    NSMapTable *invocationTable;
-#else
-    NSMutableDictionary *invocationTable;
-#endif
+    IMP invocationTable[PajeEventIdCount];
 
     /* PajeContainerType's and PajeEntityTypes's mapped by name and alias */
-    NSMutableDictionary *userTypes;
+    NSMapTable *userTypes;
 
     /* PajeContainers mapped by name and alias (used only when containerType is
        not known, like in old PajeDestroyContainer events */
-    NSMutableDictionary *userNumberToContainer;
+    //NSMutableDictionary *userNumberToContainer;
 
     NSDate *startTime;
     NSDate *endTime;
@@ -60,6 +55,12 @@
     
     /* arrays of entities related to a key */
     NSMutableDictionary *relatedEntities;
+
+    NSMutableArray *chunkInfo;
+    unsigned currentChunkNumber;
+
+    BOOL replaying;
+    BOOL lastChunkSeen;
 }
 
 - (id)initWithController:(PajeTraceController *)c;
@@ -77,14 +78,23 @@
 - (NSDate *)currentTime;
 
 - (PajeEntityType *)entityTypeWithName:(NSString *)name;
+- (id)typeForId:(const char *)typeId;
+- (void)setType:(id)type forId:(const char *)typeId;
 
 - (int)eventCount;
 
-- (void)endOfChunk;
+- (void)endOfChunkLast:(BOOL)last;
 - (void)outputChunk:(id)entity;
 
-- (void)encodeCheckPointWithCoder:(NSCoder *)coder;
-- (void)decodeCheckPointWithCoder:(NSCoder *)coder;
+- (id)chunkState;
+- (void)setChunkState:(id)state;
+- (int)currentChunkNumber;
+
+- (void)getChunksUntilTime:(NSDate *)time;
+
+- (void)notifyMissingChunk:(int)chunkNumber;
+
+- (BOOL)isReplaying;
 @end
 
 @interface PajeSimul (UserEvents)
@@ -117,6 +127,9 @@
 
 - (void)pajeStartLink:(PajeEvent *)event;
 - (void)pajeEndLink:(PajeEvent *)event;
+
+- (id)containerForId:(const char *)containerId
+                type:(PajeContainerType *)containerType;
 @end
 
 #endif
