@@ -32,6 +32,9 @@
 #include "../General/SourceCodeReference.h"
 #include "../General/NSDictionary+Additions.h"
 #include "../General/Macros.h"
+#include "../General/CStringCallBacks.h"
+
+#include "../Paje/PajeTraceController.h"
 
 #define DEFINE_STRINGS
 #include "EventNames.h"
@@ -53,13 +56,13 @@
     va_list args;
 
     va_start(args, format);
-    //NSLogv(format, args);
     [NSException raise:@"PajeSimulException" format:format arguments:args];
     va_end(args);
 }
 
 - (void)error:(NSString *)str inEvent:(PajeEvent *)event
 {
+#ifdef OLDEVENT
     PajeEvent *eventCopy;
     NSDate *evTime;
     evTime = [event valueOfFieldNamed:@"Time"];
@@ -75,6 +78,9 @@
                 [eventCopy descriptionWithLocale:[NSDictionary dictionary]
                                           indent:0]];
     [eventCopy release];
+#else
+    [self error:@"%@ in event %@", str, event];
+#endif
 }
 
 - (PajeContainer *)rootInstance
@@ -93,25 +99,23 @@
             return nil;
         }
 
-        rootContainerType = [PajeContainerType typeWithName:@"File"
-                                              containerType:nil
-                                                      event:nil];
+        rootContainerType = [PajeContainerType typeWithId:@"File"
+                                              description:@"File"
+                                            containerType:nil
+                                                    event:nil];
         rootContainer = [[SimulContainer alloc] initWithType:rootContainerType
                                                         name:rootContainerName
                                                        alias:@"0"
                                                    container:nil
                                                 creationTime:[NSDate dateWithTimeIntervalSinceReferenceDate:0]
+                                                       event:nil
                                                    simulator:self];
-        [rootContainerType addInstance:rootContainer];
+        [rootContainerType addInstance:rootContainer id1:"0" id2:"/"];
+
+        [self setType:rootContainerType forId:"0"];
+        [self setType:rootContainerType forId:"/"];
+        [self setType:rootContainerType forId:"File"];
 
-        [userTypes setObject:rootContainerType forKey:@"0"];
-        [userTypes setObject:rootContainerType forKey:@"/"];
-        [userTypes setObject:rootContainerType forKey:@"File"];
-                            
-        [userNumberToContainer setObject:rootContainer forKey:@"0"];
-        [userNumberToContainer setObject:rootContainer forKey:@"/"];
-        [userNumberToContainer setObject:rootContainer
-                                  forKey:rootContainerName];
     }
     return rootContainer;
 }
@@ -125,44 +129,11 @@
     return NSStringFromClass([self class]);
 }
 
-#ifndef MAP
-#undef NS_MESSAGE
-#ifndef NS_MESSAGE
-#ifndef GNUSTEP
-NSInvocation *invocation;
-#define NS_MESSAGE(obj, meth)                                   \
-    ((invocation = [NSInvocation invocationWithMethodSignature: \
-    [obj methodSignatureForSelector: @selector(meth)]]),        \
-    [invocation setTarget: obj],                                \
-    [invocation setSelector: @selector(meth)],                  \
-    invocation)
-#else
-#define NS_MESSAGE(target, message)                             \
-    ([[[NSInvocation alloc] initWithTarget:target                \
-    selector:@selector(message), nil] autorelease])
-#endif                          // GNUSTEP
-#define ADD_INVOCATION(name) [invocationTable  \
-                                    setObject:NS_MESSAGE(self, paje##name:) \
-                                       forKey:Paje##name##EventName]
-#else                           // NS_MESSAGE
-#define ADD_INVOCATION(name) [invocationTable  \
-                                    setObject:NS_MESSAGE(self, paje##name:nil) \
-                                       forKey:Paje##name##EventName]
-#endif                          // NS_MESSAGE
-#else /* MAP */
-#define ADD_INVOCATION(name) NSMapInsert(invocationTable, \
-                                         Paje##name##EventName, \
-                                         [self methodForSelector:@selector(paje##name:)])
-#endif /* MAP */
+#define ADD_INVOCATION(name) invocationTable[Paje##name##EventId] = \
+                                         [self methodForSelector:@selector(paje##name:)]
 
 - (void)_initInvocationTable
 {
-#ifdef MAP
-    invocationTable = NSCreateMapTable(NSIntMapKeyCallBacks/*NSObjectMapKeyCallBacks*/,
-                                       NSIntMapValueCallBacks, 50);
-#else
-    Assign(invocationTable, [NSMutableDictionary dictionary]);
-#endif
     ADD_INVOCATION(StartTrace);
     ADD_INVOCATION(DefineContainerType);
     ADD_INVOCATION(DefineEventType);
@@ -188,11 +159,15 @@ NSInvocation *invocation;
     self = [super initWithController:c];
 
     if (self != nil) {
-        userTypes = [[NSMutableDictionary alloc] init];
-        userNumberToContainer = [[NSMutableDictionary alloc] init];
+        userTypes = NSCreateMapTable(CStringMapKeyCallBacks,
+                                     NSObjectMapValueCallBacks, 50);
         relatedEntities = [[NSMutableDictionary alloc] init];
 
         [self _initInvocationTable];
+
+        chunkInfo = [[NSMutableArray alloc] init];
+        currentChunkNumber = 0;
+        currentTime = [[NSDate distantPast] retain];
     }
 
     return self;
@@ -201,17 +176,12 @@ NSInvocation *invocation;
 - (void)dealloc
 {
     [rootContainer release];
-#ifdef MAP
-    NSFreeMapTable(invocationTable);
-#else
-    [invocationTable release];
-#endif
-    [userTypes release];
-    [userNumberToContainer release];
+    NSFreeMapTable(userTypes);
     [relatedEntities release];
     [startTime release];
     [endTime release];
     [currentTime release];
+    [chunkInfo release];
     [super dealloc];
 }
 
@@ -228,15 +198,9 @@ NSInvocation *invocation;
 - (void)setCurrentTime:(NSDate *)time
 {
     Assign(currentTime, time);
-    if (endTime == nil || [time isLaterThanDate:endTime]) {
-        Assign(endTime, time);
-        if (startTime == nil) {
-            Assign(startTime, time);
-        }
-    } else if ([time isEarlierThanDate:startTime]) {
+    if (startTime == nil) {
         Assign(startTime, time);
     }
-//    NSLog(@"simul times: (%f-%f) %f", [startTime timeIntervalSinceReferenceDate], [endTime timeIntervalSinceReferenceDate], [currentTime timeIntervalSinceReferenceDate]);
 }
 
 - (NSDate *)currentTime
@@ -251,7 +215,18 @@ NSInvocation *invocation;
 
 - (PajeEntityType *)entityTypeWithName:(NSString *)name
 {
-    return [userTypes objectForKey:name];
+    return NSMapGet(userTypes, [name cString]);
+}
+
+- (id)typeForId:(const char *)typeId
+{
+    if (typeId == NULL) return nil;
+    return NSMapGet(userTypes, typeId);
+}
+
+- (void)setType:(id)type forId:(const char *)typeId
+{
+    NSMapInsert(userTypes, strdup(typeId), type);
 }
 
 - (void)inputEntity:(PajeEvent *)event
@@ -260,67 +235,65 @@ NSInvocation *invocation;
 
     [self rootContainer];
 
-{
+#ifdef SUPPORT_FOR_SOURCE_REFERENCES
+    // set event's filename -- not supported
     NSString *filename;
-        // set event's filename
-//    file = [event objectForKey:@"File"];
-//    if (file)
-//        filename = [fileToFilename objectForKey:file];
-//    else
-        filename = [event objectForKey:@"FileName"];
-    if (filename) {
-        id line;
+    filename = [event stringForFieldId:PajeFileFieldId];
+    if (filename != nil) {
         int lineNumber;
         SourceCodeReference *sourceRef;
-//        NSMutableDictionary *refToEvents;
 
-        line = [event objectForKey:@"Line"];
-        if (line == nil)
-            line = [event objectForKey:@"LineNumber"];
-        lineNumber = [line intValue];
+        lineNumber = [event intForFieldId:PajeLineFieldId];
         sourceRef = [SourceCodeReference referenceToFilename:filename
                                                   lineNumber:lineNumber];
-//        refToEvents = [filenameToReferences objectForKey:filename];
-//        [refToEvents addObject:event forKey:sourceRef];
-        [event removeObjectForKey:@"File"];
-        [event removeObjectForKey:@"FileName"];
-        [event removeObjectForKey:@"Line"];
-        [event removeObjectForKey:@"LineNumber"];
-        [event setObject:sourceRef forKey:@"FileReference"];
     }
-
-}
+#endif
 
     // set the current time, if event has one
     time = [event time];
     if (time != nil) {
         [self setCurrentTime:time];
     }
+
     // invoke the method that simulates this event
-#ifndef MAP
-    NSInvocation *invocation;
-    invocation = [invocationTable objectForKey:[event pajeEventName]];
-    if (invocation) {
-        [invocation setArgument:&event atIndex:2];
-        [invocation invoke];
+    int eventId;
+    eventId = [event pajeEventId];
+    IMP f = NULL;
+    if (eventId >= 0 
+           && eventId < (sizeof invocationTable / sizeof(invocationTable[0]))) {
+        f = invocationTable[eventId];
     }
-#else
-    void (*f)(id, SEL, id);
-    f = NSMapGet(invocationTable, [event pajeEventName]);
     if (f != NULL) {
         f(self, 0, event);
     } else {
-        NSLog(@"Unknown event \"%@\"", [event pajeEventName]);
+        NSLog(@"Unknown event \"%@\"", event);
     }
-#endif
     
     eventCount++;
 }
 
+
+- (BOOL)canEndChunkBefore:(PajeEvent *)event
+{
+    id time;
+
+    time = [event time];
+    if (time != nil 
+          && [currentTime isLaterThanDate:[NSDate distantPast]]
+          && [currentTime isLaterThanDate:[NSDate dateWithTimeIntervalSinceReferenceDate:0]] 
+          && [time isLaterThanDate:currentTime]) {
+        return YES;
+    }
+
+    [self inputEntity:event];
+    return NO;
+}
+
+
 - (void)_resetContainers
 {
-    [[userNumberToContainer allValues]
-        makeObjectsPerformSelector:@selector(reset)];
+    //[[userNumberToContainer allValues]
+    //    makeObjectsPerformSelector:@selector(reset)];
 }
 
 - (void)addRelatedEntity:(id)entity toKey:(id)key
@@ -359,77 +332,208 @@ NSInvocation *invocation;
 
 - (void)outputChunk:(id)entity
 {
+    if (replaying) {
+        return;
+    }
     [super outputEntity:entity];
 }
 
 
-- (void)endOfChunkInContainer:(SimulContainer *)container
+- (void)startChunkInContainer:(SimulContainer *)container
 {
     NSEnumerator *subContainerEnumerator;
     SimulContainer *subContainer;
 
-    [container endOfChunk];
+    [container startChunk];
 
     subContainerEnumerator = [[container subContainers] objectEnumerator];
     while ((subContainer = [subContainerEnumerator nextObject]) != nil) {
-        [self endOfChunkInContainer:subContainer];
+        [self startChunkInContainer:subContainer];
     }
 }
 
-- (void)endOfChunk
+- (void)endOfChunkInContainer:(SimulContainer *)container
+                         last:(BOOL)last
 {
-    [self endOfChunkInContainer:rootContainer];
-}
+    NSEnumerator *subContainerEnumerator;
+    SimulContainer *subContainer;
 
-
-- (void)encodeCheckPointWithCoder:(NSCoder *)coder
-{
-    NSEnumerator *containerEnum;
-    SimulContainer *container;
-    NSMutableSet *c;
-
-    c = [[NSMutableSet alloc] init];
-
-//    [coder encodeObject:startTime];
-//    [coder encodeObject:endTime];
-    [coder encodeObject:currentTime];
-    [coder encodeValueOfObjCType:@encode(int) at:&eventCount];
-    // FIXME: should be saved by type
-    containerEnum = [[userNumberToContainer allValues] objectEnumerator];
-    while ((container = [containerEnum nextObject]) != nil) {
-        if (![c containsObject:[container alias]]) {
-            [c addObject:[container alias]];
-            [coder encodeObject:[container alias]];
-            [container encodeCheckPointWithCoder:coder];
-        }
-    }
-    [coder encodeObject:nil];
-    [c release];
-}
-
-- (void)decodeCheckPointWithCoder:(NSCoder *)coder
-{
-    NSString *containerName;
+    [container endOfChunkLast:last];
     
-//    Assign(startTime, [coder decodeObject]);
-//    Assign(endTime, [coder decodeObject]);
-    Assign(currentTime, [coder decodeObject]);
-    [coder decodeValueOfObjCType:@encode(int) at:&eventCount];
+    if (endTime == nil || [endTime isEarlierThanDate:currentTime]) {
+        Assign(endTime, currentTime);
+    }
+
+    subContainerEnumerator = [[container subContainers] objectEnumerator];
+    while ((subContainer = [subContainerEnumerator nextObject]) != nil) {
+        [self endOfChunkInContainer:subContainer last:last];
+    }
+}
+
+- (void)emptyChunk:(int)chunkNumber
+       inContainer:(SimulContainer *)container
+{
+    NSEnumerator *subContainerEnumerator;
+    SimulContainer *subContainer;
+
+    [container emptyChunk:chunkNumber];
+    
+    subContainerEnumerator = [[container subContainers] objectEnumerator];
+    while ((subContainer = [subContainerEnumerator nextObject]) != nil) {
+        [self emptyChunk:chunkNumber inContainer:subContainer];
+    }
+}
+
+- (void)addStateOfContainer:(SimulContainer *)container
+                     toDict:(NSMutableDictionary *)dict
+{
+    NSEnumerator *subContainerEnumerator;
+    SimulContainer *subContainer;
+
+    [dict setObject:[container chunkState] forKey:[container alias]];
+    
+    subContainerEnumerator = [[container subContainers] objectEnumerator];
+    while ((subContainer = [subContainerEnumerator nextObject]) != nil) {
+        [self addStateOfContainer:subContainer toDict:dict];
+    }
+}
+
+- (id)chunkState
+{
+    //NSEnumerator *containerEnum;
+    //SimulContainer *container;
+    NSMutableDictionary *state;
+
+    state = [NSMutableDictionary dictionary];
+
+    [state setObject:currentTime forKey:@"_currentTime"];
+    [state setObject:[NSNumber numberWithInt:eventCount] forKey:@"_eventCount"];
+
+    // FIXME: should be saved by type
+    //containerEnum = [[userNumberToContainer allValues] objectEnumerator];
+    //while ((container = [containerEnum nextObject]) != nil) {
+    //    id key = [container alias];
+    //    if (nil == [state objectForKey:key]) {
+    //        [state setObject:[container chunkState] forKey:key];
+    //    }
+    //}
+    [self addStateOfContainer:(SimulContainer *)[self rootContainer]
+                       toDict:state];
+    
+    return state;
+}
+
+- (void)setChunkState:(id)obj
+{
+    NSDictionary *state = (NSDictionary *)obj;
+
+    [self setCurrentTime:[state objectForKey:@"_currentTime"]];
+    eventCount = [[state objectForKey:@"_eventCount"] intValue];
 
     [self _resetContainers];
 
-    while ((containerName = [coder decodeObject]) != nil) {
+    NSEnumerator *keyEnum;
+    id key;
+    keyEnum = [state keyEnumerator];
+    while ((key = [keyEnum nextObject]) != nil) {
         SimulContainer *container;
-        container = [userNumberToContainer objectForKey:containerName];
+        //container = [userNumberToContainer objectForKey:key];
+        container = [self containerForId:[key cString] type:nil];
         if (container == nil) {
-        NSLog(@"known: %@", userNumberToContainer);
-            [self error:@"Decoding unknown container '%@'", containerName];
+            if (![key isEqual:@"_currentTime"]
+                && ![key isEqual:@"_eventCount"]) {
+                [self error:@"Decoding unknown container '%@'", key];
+            }
+        } else {
+            [container setChunkState:[state objectForKey:key]];
         }
-        NSDebugMLLog(@"enc", @"found container %@", containerName);
-        [container decodeCheckPointWithCoder:coder];
     }
 }
 
+- (int)currentChunkNumber
+{
+    return currentChunkNumber;
+}
+
+- (void)startChunk:(int)chunkNumber
+{
+    if (chunkNumber != currentChunkNumber) {
+        if (chunkNumber >= [chunkInfo count]) {
+            // cannot position in an unread place
+            [self error:@"Cannot start unknown chunk"];
+        }
+
+        [self setChunkState:[chunkInfo objectAtIndex:chunkNumber]];
+
+        currentChunkNumber = chunkNumber;
+    } else {
+        // let's register the first chunk position
+        if ([chunkInfo count] == 0) {
+
+            [chunkInfo addObject:[self chunkState]];
+
+        }
+    }
+
+    if (!lastChunkSeen && currentChunkNumber == [chunkInfo count] - 1) {
+        replaying = NO;
+    } else {
+        replaying = YES;
+    }
+
+    [self startChunkInContainer:rootContainer];
+
+    // keep the ball rolling (tell other components)
+    [super startChunk:chunkNumber];
+}
+
+- (BOOL)isReplaying
+{
+    return replaying;
+}
+
+
+- (void)endOfChunkLast:(BOOL)last
+{
+    [self endOfChunkInContainer:rootContainer last:last];
+
+    if (!last) {
+        currentChunkNumber++;
+        // if we're at the end of the known world, let's register its position
+        if (currentChunkNumber == [chunkInfo count]) {
+
+            [chunkInfo addObject:[self chunkState]];
+
+        }
+    } else {
+        lastChunkSeen = YES;
+    }
+    [super endOfChunkLast:(BOOL)last];
+}
+
+- (void)emptyChunk:(int)chunkNumber
+{
+    [self emptyChunk:chunkNumber inContainer:rootContainer];
+}
+
+- (void)notifyMissingChunk:(int)chunkNumber
+{
+    [controller missingChunk:chunkNumber];
+}
+
+- (void)getChunksUntilTime:(NSDate *)time
+{
+    while ([currentTime isEarlierThanDate:time]) {
+        int chunkNumber = [chunkInfo count] - 1;
+        [self notifyMissingChunk:chunkNumber];
+        // if no chunk has been read, give up
+        if ([chunkInfo count] - 1 == chunkNumber) {
+            break;
+        }
+    }
+}
+
+
 - (id)copyWithZone:(NSZone *)zone
 {
     return [self retain];
