@@ -31,16 +31,20 @@
 #include "../General/PajeEvent.h"
 #include "SimulChunk.h"
 
+@class PajeSimul;
+
 @interface SimulContainer : PajeContainer
 {
     NSDate *creationTime;
     NSDate *lastTime;
-    id simulator;
+    PajeSimul *simulator;
     NSString *alias;
     NSMutableDictionary *userEntities; // key = entityType
     NSMutableDictionary *minValues; // key = entityType
     NSMutableDictionary *maxValues; // key = entityType
+    NSDictionary *extraFields;
     int logicalTime;
+    BOOL isActive;
 }
 
 + (SimulContainer *)containerWithType:(PajeEntityType *)type
@@ -48,12 +52,14 @@
                                 alias:(NSString *)a
                            container:(PajeContainer *)newcontainer
                         creationTime:(NSDate *)time
+                               event:(PajeEvent *)event
                            simulator:(id)simul;
 - (id)initWithType:(PajeEntityType *)type
               name:(NSString *)n
              alias:(NSString *)a
          container:(PajeContainer *)c
       creationTime:(NSDate *)time
+             event:(PajeEvent *)event
          simulator:(id)simul;
 
 - (NSString *)alias;
@@ -63,10 +69,13 @@
 
 - (void)stopWithEvent:(PajeEvent*)event;
 
+- (BOOL)isStopped;
+- (BOOL)isActive;
+
 - (void)setLastTime:(NSDate *)time;
 
 - (SimulChunk *)chunkOfType:(id)type;
-
+- (SimulChunk *)previousChunkOfType:(id)type;
 
 - (void)newEventWithType:(id)type
                    value:(id)value
@@ -81,13 +90,13 @@
                  withEvent:(PajeEvent *)event;
 
 - (void)setUserVariableOfType:(PajeVariableType *)entityType
-                      toValue:(id)entityName
+                toDoubleValue:(double)value
                     withEvent:(PajeEvent *)event;
 - (void)addUserVariableOfType:(PajeVariableType *)entityType
-                        value:(id)entityName
+                  doubleValue:(double)value
                     withEvent:(PajeEvent *)event;
 - (void)subUserVariableOfType:(PajeVariableType *)entityType
-                        value:(id)entityName
+                  doubleValue:(double)value
                     withEvent:(PajeEvent *)event;
 
 - (void)startUserLinkOfType:(PajeEntityType *)entityType
@@ -101,18 +110,21 @@
                       key:(id)key
                 withEvent:(PajeEvent*)event;
 
-- (NSNumber *)minValueForEntityType:(PajeEntityType *)entityType;
-- (NSNumber *)maxValueForEntityType:(PajeEntityType *)entityType;
+- (double)minValueForEntityType:(PajeEntityType *)entityType;
+- (double)maxValueForEntityType:(PajeEntityType *)entityType;
 
 - (int)logicalTime;
 - (void)setLogicalTime:(int)lt;
 
-- (void)endOfChunk;
+- (void)startChunk;
+- (void)endOfChunkLast:(BOOL)last;
+
+- (void)emptyChunk:(int)chunkNumber;
 
 - (void)reset;
-- (void)encodeCheckPointWithCoder:(NSCoder *)coder;
-- (void)decodeCheckPointWithCoder:(NSCoder *)coder;
 
+- (id)chunkState;
+- (void)setChunkState:(id)state;
 @end
 
 #endif
