@@ -34,7 +34,8 @@
 
 @interface UserEvent : PajeEntity
 {
-    PajeEvent *event;
+    NSDictionary *extraFields;
+    NSDate *time;
 }
 
 + (UserEvent *)eventWithType:(PajeEntityType *)type
@@ -47,6 +48,7 @@
              event:(PajeEvent *)e;
 - (void)dealloc;
 
+- (void)setEvent:(PajeEvent *)e;
 - (NSDate *)time;
 @end
 
