@@ -108,7 +108,7 @@
                sourceEvent:(PajeEvent *)e
 {
     sourceContainer = sc;   // not retained
-    Assign(event, e);
+    [self setEvent:e];
 }
 
 - (void)setDestContainer:(PajeContainer *)dc
@@ -120,15 +120,15 @@
 
 - (BOOL)canBeEndedWithValue:(id)v key:(id)k
 {
-    return ((destContainer == nil)
-            && [[self value] isEqual:v]
+    return (/*(destContainer == nil)
+            && */[[self value] isEqual:v]
             && [key isEqual:k]);
 }
 
 - (BOOL)canBeStartedWithValue:(id)v key:(id)k
 {
-    return ((sourceContainer == nil)
-            && [[self value] isEqual:v]
+    return (/*(sourceContainer == nil)
+            && */[[self value] isEqual:v]
             && [key isEqual:k]);
 }
 
@@ -193,8 +193,8 @@
 {
 //if (startLogicalTime != nil)
 //    return startLogicalTime;
-    if (event != nil) {
-        return [event time];
+    if (time != nil) {
+        return [super startTime];
     } else if (sourceContainer != nil) {
         return [sourceContainer endTime];
     } else {
@@ -218,4 +218,9 @@
 {
     return endLogicalTime;
 }
+
+- (id) key
+{
+    return key;
+}
 @end
