@@ -141,7 +141,7 @@
                startTime:(NSDate *)startTime
                  endTime:(NSDate *)endTime
               enumerator:(NSEnumerator *)enumerator
-              nameFilter:(NSSet *)filter
+             valueFilter:(NSSet *)filter
 {
     self = [self initWithEntityType:et
                           container:cont
@@ -150,7 +150,7 @@
     if (self) {
         PajeEntity *entity;
         while ((entity = [enumerator nextObject]) != nil) {
-            if (![filter containsObject:[entity name]]) {
+            if (![filter containsObject:[entity value]]) {
                 [self addEntity:entity];
             }
         }
@@ -229,8 +229,11 @@
                                    toTime:(NSDate *)t2
 {
     BusyArrayEnumerator *enumerator;
-    unsigned i1, i2;
+    NSUInteger i1, i2;
     i1 = [self indexOfLastObjectNotAfterValue:t1];
+    if (i1 == NSNotFound) {
+        i1 = 0;
+    }
     i2 = [self indexOfFirstObjectNotBeforeValue:t2];
     enumerator = [[BusyArrayEnumerator alloc] initWithArray:self
                                                 entityClass:c
@@ -239,6 +242,25 @@
     return [enumerator autorelease];
 }
 
+- (NSEnumerator *)completeObjectEnumeratorOfClass:(Class)c
+                                         fromTime:(NSDate *)t1
+                                           toTime:(NSDate *)t2
+{
+    BusyArrayEnumerator *enumerator;
+    NSUInteger i1, i2;
+    i1 = [self indexOfFirstObjectNotBeforeValue:t1];
+    i2 = [self indexOfLastObjectNotAfterValue:t2];
+    if (i2 != NSNotFound && i2 != 0) {
+        enumerator = [[BusyArrayEnumerator alloc] initWithArray:self
+                                                    entityClass:c
+                                                     firstIndex:i1
+                                                      lastIndex:i2 - 1];
+        return [enumerator autorelease];
+    } else {
+        return nil;
+    }
+}
+
 
 - (NSEnumerator *)reverseObjectEnumeratorOfClass:(Class)c
 {
@@ -251,8 +273,11 @@
                                           toTime:(NSDate *)t2
 {
     BusyArrayEnumerator *enumerator;
-    unsigned i1, i2;
+    NSUInteger i1, i2;
     i1 = [self indexOfLastObjectNotAfterValue:t1];
+    if (i1 == NSNotFound) {
+        i1 = 0;
+    }
     i2 = [self indexOfFirstObjectNotBeforeValue:t2];
     enumerator = [[BusyArrayReverseEnumerator alloc] initWithArray:self
                                                        entityClass:c
