@@ -33,7 +33,7 @@
     if (self) {
 
         // load the interface. it initializes view.
-        if (![NSBundle loadNibNamed:@"BusyNode" owner:self])
+        if (![NSBundle loadNibNamed:@"ReductionFilter" owner:self])
             NSRunAlertPanel(NSStringFromClass([self class]),
                             @"Couldn't load interface file",
                             nil, nil, nil);
@@ -150,8 +150,9 @@
     [entityNamePopUp removeAllItems];
     typeEnumerator = [reduceEntityTypes objectEnumerator];
     while ((type = [typeEnumerator nextObject]) != nil) {
-        [entityNamePopUp addItemWithTitle:[type name]];
-        [[entityNamePopUp itemWithTitle:[type name]] setRepresentedObject:type];
+        [entityNamePopUp addItemWithTitle:[type description]];
+        [[entityNamePopUp itemWithTitle:[type description]]
+                                    setRepresentedObject:type];
 	ct++;
     }
     if (ct == 0) {
@@ -161,7 +162,7 @@
         [entityNamePopUp setEnabled:YES];
     }
     if (selectedType != nil)
-        [entityNamePopUp selectItemWithTitle:[selectedType name]];
+        [entityNamePopUp selectItemWithTitle:[selectedType description]];
     if ([entityNamePopUp selectedItem] == nil
         && [entityNamePopUp numberOfItems] > 0)
         [entityNamePopUp selectItemAtIndex:0];
@@ -182,8 +183,10 @@
     subenum = [[self allEntityTypes] objectEnumerator];
     while ((entityType = [subenum nextObject]) != nil) {
         if (![self isContainerEntityType:entityType]) {
-            [entityTypePopUp addItemWithTitle:[entityType name]];
-            [[entityTypePopUp itemWithTitle:[entityType name]]
+            NSString *title;
+            title = [self descriptionForEntityType:entityType];
+            [entityTypePopUp addItemWithTitle:title];
+            [[entityTypePopUp itemWithTitle:title]
                                        setRepresentedObject:entityType];
             ct++;
         }
@@ -194,7 +197,9 @@
     } else {
         [entityTypePopUp setEnabled:YES];
     }
-    [entityTypePopUp selectItemWithTitle:[[type entityTypeToReduce] name]];
+    NSString *selectedTitle;
+    selectedTitle = [self descriptionForEntityType:[type entityTypeToReduce]];
+    [entityTypePopUp selectItemWithTitle:selectedTitle];
 }
 
 - (void)calcGroupPopUp
@@ -210,7 +215,9 @@
     while ((parentEntityType = [self containerTypeForType:entityType]) != nil) {
         if (parentEntityType == entityType) break;
         entityType = parentEntityType;
-        [groupByPopUp insertItemWithTitle:[entityType name] atIndex:0];
+        NSString *title;
+        title = [self descriptionForEntityType:entityType];
+        [groupByPopUp insertItemWithTitle:title atIndex:0];
         [[groupByPopUp itemAtIndex:0] setRepresentedObject:entityType];
 	ct++;
     }
@@ -220,7 +227,10 @@
     } else {
         [groupByPopUp setEnabled:YES];
     }
-    [groupByPopUp selectItemWithTitle:[[self containerTypeForType:type] name]];
+    NSString *selectedTitle;
+    selectedTitle = [self descriptionForEntityType:
+                                              [self containerTypeForType:type]];
+    [groupByPopUp selectItemWithTitle:selectedTitle];
 }
 
 - (void)calcReduceModePopUp
@@ -321,7 +331,7 @@
                                              component:self];
         [newEntityType setEntityClass:[selectedEntityType entityClass]];
         [newEntityType setEntityTypeToReduce:[selectedEntityType entityTypeToReduce]];
-        [newEntityType addNamesToFilter:[[selectedEntityType filterNames] allObjects]];        
+        [newEntityType addValuesToFilter:[[selectedEntityType filterValues] allObjects]];        
     } else {
         newEntityType = [ReduceEntityType typeWithName:newEntityName
                                          containerType:(PajeContainerType *)[self rootEntityType]
@@ -331,12 +341,12 @@
     }
     [reduceEntityTypes addObject:newEntityType];
     [entityTypesDictionary setObject:[newEntityType dictionaryForDefaults]
-                              forKey:[newEntityType name]];
+                              forKey:[newEntityType description]];
     [self registerDefaults];
     [self addToHierarchy:newEntityType];
 
     [self calcEntityNamePopUp];
-    [entityNamePopUp selectItemWithTitle:[newEntityType name]];
+    [entityNamePopUp selectItemWithTitle:[newEntityType description]];
     [self entityNamePopUpChanged:self];
     [super hierarchyChanged];
 }
@@ -349,7 +359,9 @@
         NSBeep();
         return;
     }
-    [entityTypesDictionary removeObjectForKey:[entityType name]];
+    [entityTypesDictionary removeObjectForKey:[entityType description]];
+    [reduceEntityTypes removeObject:entityType];
+    [self calcHierarchy];
     [self registerDefaults];
     [self hierarchyChanged];
 }
@@ -360,20 +372,23 @@
     NSString *oldName;
     NSString *newName;
     entityType = [[entityNamePopUp selectedItem] representedObject];
-    oldName = [entityType name];
+    oldName = [entityType description];
     newName = [entityNameField stringValue];
     if ((oldName == nil) || (newName == nil) || [newName isEqual:oldName]) {
         NSBeep();
         return;
     }
-    [entityType setName:newName];
+    [reduceEntityTypes removeObject:entityType];
+    [entityType setDescription:newName];
+    [reduceEntityTypes addObject:entityType];
     [entityTypesDictionary removeObjectForKey:oldName];
     [entityTypesDictionary setObject:[entityType dictionaryForDefaults]
                               forKey:newName];
+    [self calcHierarchy];
     [self registerDefaults];
     [[entityNamePopUp selectedItem] setTitle:newName];
     [entityNamePopUp synchronizeTitleAndSelectedItem];
-    [super hierarchyChanged];
+//    [super hierarchyChanged];
 }
 
 - (IBAction)entityNamePopUpChanged:(id)sender
@@ -398,7 +413,7 @@
     if (![[type entityTypeToReduce] isEqual:newEntityTypeToReduce]) {
         [type setEntityTypeToReduce:newEntityTypeToReduce];
         [entityTypesDictionary setObject:[type dictionaryForDefaults]
-                                  forKey:[type name]];
+                                  forKey:[type description]];
         [self registerDefaults];
         [self calcGroupPopUp];
         [self refreshMatrix];
@@ -420,7 +435,7 @@
     if (![newContainerType isEqual:[type containerType]]) {
         [type setContainerType:newContainerType];
         [entityTypesDictionary setObject:[type dictionaryForDefaults]
-                                  forKey:[type name]];
+                                  forKey:[type description]];
         [self registerDefaults];
         [self refreshMatrix];
         [self calcHierarchy];
@@ -445,7 +460,7 @@
     [type setEntityClass:entityClass];
     
     [entityTypesDictionary setObject:[type dictionaryForDefaults]
-                              forKey:[type name]];
+                              forKey:[type description]];
     [self registerDefaults];
 
     [super dataChangedForEntityType:type];
@@ -467,12 +482,13 @@
         NSBeep();
         return;
     }
-    if ([cell state])
-        [type removeNameFromFilter:[cell representedObject]];
-    else
-        [type addNameToFilter:[cell representedObject]];
+    if ([cell state]) {
+        [type removeValueFromFilter:[cell representedObject]];
+    } else {
+        [type addValueToFilter:[cell representedObject]];
+    }
     [entityTypesDictionary setObject:[type dictionaryForDefaults]
-                              forKey:[type name]];
+                              forKey:[type description]];
     [self registerDefaults];
 
     [super dataChangedForEntityType:type];
@@ -487,7 +503,7 @@
     ReduceEntityType *type;
 
     type = [[entityNamePopUp selectedItem] representedObject];
-    allArray = [inputComponent allNamesForEntityType:[type entityTypeToReduce]];
+    allArray = [inputComponent allValuesForEntityType:[type entityTypeToReduce]];
 
     n = [allArray count];
     [nameMatrix renewRows:n columns:1];
@@ -496,7 +512,7 @@
         cell = [nameMatrix cellAtRow:i column:0];
         [cell setTitle:name];
         [cell setRepresentedObject:name];
-        [cell setState:![[type filterNames] containsObject:name]];
+        [cell setState:![[type filterValues] containsObject:name]];
     }
     [nameMatrix sizeToFit];
     [nameMatrix setNeedsDisplay:YES];
@@ -522,12 +538,13 @@
                                      toTime:(NSDate *)end
                                 minDuration:(double)minDuration
 {
-    if (![entityType isKindOfClass:[ReduceEntityType class]])
-        return [inputComponent enumeratorOfEntitiesTyped:entityType
-                                             inContainer:container
-                                                fromTime:start
-                                                  toTime:end
-                                             minDuration:minDuration];
+    if (![entityType isKindOfClass:[ReduceEntityType class]]) {
+        return [super enumeratorOfEntitiesTyped:entityType
+                                    inContainer:container
+                                       fromTime:start
+                                         toTime:end
+                                    minDuration:minDuration];
+    }
 
     return [(ReduceEntityType *)entityType
                         enumeratorOfEntitiesInContainer:container
@@ -536,6 +553,26 @@
                                             minDuration:minDuration];
 }
 
+- (NSEnumerator *)enumeratorOfCompleteEntitiesTyped:(PajeEntityType *)entityType
+                                        inContainer:(PajeContainer *)container
+                                           fromTime:(NSDate *)start
+                                             toTime:(NSDate *)end
+                                        minDuration:(double)minDuration
+{
+    if (![entityType isKindOfClass:[ReduceEntityType class]]) {
+        return [super enumeratorOfCompleteEntitiesTyped:entityType
+                                            inContainer:container
+                                               fromTime:start
+                                                 toTime:end
+                                            minDuration:minDuration];
+    }
+
+    return [(ReduceEntityType *)entityType
+                        enumeratorOfCompleteEntitiesInContainer:container
+                                                       fromTime:start
+                                                         toTime:end
+                                                    minDuration:minDuration];
+}
 
 - (BOOL)canHighlightEntity:(PajeEntity *)entity
 {
@@ -549,52 +586,74 @@
 - (PajeDrawingType)drawingTypeForEntityType:(PajeEntityType *)entityType
 {
     if ([entityType isKindOfClass:[ReduceEntityType class]])
-        return [entityType drawingType];
+        return [(ReduceEntityType *)entityType drawingType];
     else
         return [super drawingTypeForEntityType:entityType];
 }
 
-- (NSNumber *)valueForEntity:(PajeEntity *)entity
+- (double)doubleValueForEntity:(PajeEntity *)entity
 {
     if ([entity isKindOfClass:[ReduceEntity class]]) {
-        return [(ReduceEntity *)entity value];
+        return [(ReduceEntity *)entity doubleValue];
     } else {
-        return [inputComponent valueForEntity:entity];
+        return [inputComponent doubleValueForEntity:entity];
     }
 }
 
-- (NSNumber *)minValueForEntityType:(PajeEntityType *)entityType
+- (double)minValueForEntity:(PajeEntity *)entity
 {
-    if ([entityType isKindOfClass:[ReduceEntityType class]])
+    if ([entity isKindOfClass:[ReduceEntity class]]) {
+        return [(ReduceEntity *)entity minValue];
+    } else {
+        return [inputComponent minValueForEntity:entity];
+    }
+}
+
+- (double)maxValueForEntity:(PajeEntity *)entity
+{
+    if ([entity isKindOfClass:[ReduceEntity class]]) {
+        return [(ReduceEntity *)entity maxValue];
+    } else {
+        return [inputComponent maxValueForEntity:entity];
+    }
+}
+
+- (double)minValueForEntityType:(PajeEntityType *)entityType
+{
+    if ([entityType isKindOfClass:[ReduceEntityType class]]) {
         return [(ReduceEntityType *)entityType minValue];
-    else
+    } else {
         return [super minValueForEntityType:entityType];
+    }
 }
 
-- (NSNumber *)maxValueForEntityType:(PajeEntityType *)entityType
+- (double)maxValueForEntityType:(PajeEntityType *)entityType
 {
-    if ([entityType isKindOfClass:[ReduceEntityType class]])
+    if ([entityType isKindOfClass:[ReduceEntityType class]]) {
         return [(ReduceEntityType *)entityType maxValue];
-    else
+    } else {
         return [super maxValueForEntityType:entityType];
+    }
 }
 
-- (NSNumber *)minValueForEntityType:(PajeEntityType *)entityType
-                        inContainer:(PajeContainer *)container
+- (double)minValueForEntityType:(PajeEntityType *)entityType
+                    inContainer:(PajeContainer *)container
 {
-    if ([entityType isKindOfClass:[ReduceEntityType class]])
+    if ([entityType isKindOfClass:[ReduceEntityType class]]) {
         return [(ReduceEntityType *)entityType minValue];
-    else
+    } else {
         return [super minValueForEntityType:entityType inContainer:container];
+    }
 }
 
-- (NSNumber *)maxValueForEntityType:(PajeEntityType *)entityType
-                        inContainer:(PajeContainer *)container
+- (double)maxValueForEntityType:(PajeEntityType *)entityType
+                    inContainer:(PajeContainer *)container
 {
-    if ([entityType isKindOfClass:[ReduceEntityType class]])
+    if ([entityType isKindOfClass:[ReduceEntityType class]]) {
         return [(ReduceEntityType *)entityType maxValue];
-    else
+    } else {
         return [super maxValueForEntityType:entityType inContainer:container];
+    }
 }
 
 - (id)configuration
