@@ -22,6 +22,8 @@
 #include "../General/Macros.h"
 #include "../General/NSUserDefaults+Additions.h"
 
+#include <math.h>
+
 @implementation ReduceEntityType
 
 + (ReduceEntityType *)typeFromDictionary:(NSDictionary *)dict
@@ -56,13 +58,14 @@
     [newType setEntityTypeToReduce:red];
 
     reductionClass = NSClassFromString([dict objectForKey:@"ReductionClass"]);
-    if (reductionClass == Nil)
+    if (reductionClass == Nil) {
         reductionClass = [CountReduceEntity class];
+    }
     [newType setEntityClass:reductionClass];
 
-    en = [[dict objectForKey:@"NamesToFilter"] objectEnumerator];
+    en = [[dict objectForKey:@"ValuesToFilter"] objectEnumerator];
     while ((nameToFilter = [en nextObject]) != nil) {
-        [newType addNameToFilter:nameToFilter];
+        [newType addValueToFilter:nameToFilter];
     }
     return newType;
 }
@@ -81,34 +84,20 @@
          component:(PajeFilter *)comp
 {
     // super adds self as a subtype of containertype.
-    // init has been copied from super so as to not do that
-    if (self == [super init]) {
-        NSColor *c;
-        Assign(name, n);
-        containerType = type;
-//	[containerType addContainedType:self];
-        c = [[NSUserDefaults standardUserDefaults] colorForKey:[name stringByAppendingString:@" Color"]];
-        if (c == nil) {
-            c = [NSColor blackColor];
-        }
-        Assign(color, c);
-        fieldNames = [[NSMutableSet alloc] init];
-
-        component = comp;
-        filterNames = [[NSMutableSet alloc] init];
-    }
-    return self;
-    self = [super initWithName:n containerType:type event:nil];
+    // call init with nil as containerType so as to not do that;
+    // initialize containerType after it.
+    self = [super initWithId:n description:n containerType:nil event:nil];
     if (self != nil) {
+        containerType = type;
         component = comp;
-        filterNames = [[NSMutableSet alloc] init];
+        filterValues = [[NSMutableSet alloc] init];
     }
     return self;
 }
 
 - (void)dealloc
 {
-    Assign(filterNames, nil);
+    Assign(filterValues, nil);
     Assign(entityTypeToReduce, nil);
     Assign(array, nil);
     [super dealloc];
@@ -122,17 +111,17 @@
 - (NSDictionary *)dictionaryForDefaults
 {
     return [NSDictionary dictionaryWithObjectsAndKeys:
-        [self name], @"EntityName",
-        [[self containerType] name], @"ContainerType",
-        [[self entityTypeToReduce] name], @"EntityTypeToReduce",
-        [filterNames allObjects], @"NamesToFilter",
+        [self description], @"EntityName",
+        [[self containerType] description], @"ContainerType",
+        [[self entityTypeToReduce] description], @"EntityTypeToReduce",
+        [filterValues allObjects], @"ValuesToFilter",
         NSStringFromClass(entityClass), @"ReductionClass",
         nil];
 }
 
-- (void)setName:(NSString *)n
+- (void)setDescription:(NSString *)n
 {
-    Assign(name, n);
+    Assign(description, n);
 }
 
 - (void)setContainerType:(PajeContainerType *)newContainerType
@@ -141,27 +130,25 @@
 
     Assign(containerType, newContainerType);
 
-    defaultName = [[self name] stringByAppendingString:@"ContainerType"];
-    [[NSUserDefaults standardUserDefaults] setObject:[containerType name]
+    defaultName = [[self description] stringByAppendingString:@"ContainerType"];
+    [[NSUserDefaults standardUserDefaults] setObject:[containerType description]
                                               forKey:defaultName];
 
-    Assign(minValue, nil);
-    Assign(maxValue, nil);
+    minValue = HUGE_VAL;
+    maxValue = -HUGE_VAL;
     Assign(array, nil);
 }
 
 - (void)setEntityClass:(Class)c
 {
     entityClass = c;
-    Assign(minValue, nil);
-    Assign(maxValue, nil);
+    minValue = HUGE_VAL;
+    maxValue = -HUGE_VAL;
     if (array != nil) {
         [entityClass getMinValue:&minValue
                         maxValue:&maxValue
                         forArray:array
                    pajeComponent:component];
-        [minValue retain];
-        [maxValue retain];
     }
 }
 
@@ -179,10 +166,10 @@
 {
     Assign(entityTypeToReduce, newEntityTypeToReduce);
 
-    [filterNames removeAllObjects];
+    [filterValues removeAllObjects];
     Assign(array, nil);
-    Assign(minValue, nil);
-    Assign(maxValue, nil);
+    minValue = HUGE_VAL;
+    maxValue = -HUGE_VAL;
 }
 
 - (PajeEntityType *)entityTypeToReduce
@@ -190,32 +177,33 @@
     return entityTypeToReduce;
 }
 
-- (void)addNameToFilter:(NSString *)n
+- (void)addValueToFilter:(id)value
 {
-    [filterNames addObject:n];
+    [filterValues addObject:value];
     Assign(array, nil);
-    Assign(minValue, nil);
-    Assign(maxValue, nil);
+    minValue = HUGE_VAL;
+    maxValue = -HUGE_VAL;
 }
 
-- (void)addNamesToFilter:(NSArray *)names
+- (void)addValuesToFilter:(NSArray *)values
 {
-    [filterNames addObjectsFromArray:names];
+    [filterValues addObjectsFromArray:values];
     Assign(array, nil);
-    Assign(minValue, nil);
-    Assign(maxValue, nil);
+    minValue = HUGE_VAL;
+    maxValue = -HUGE_VAL;
 }
 
-- (void)removeNameFromFilter:(NSString *)n
+- (void)removeValueFromFilter:(id)value
 {
-    [filterNames removeObject:n];
+    [filterValues removeObject:value];
     Assign(array, nil);
-    Assign(minValue, nil);
-    Assign(maxValue, nil);
+    minValue = HUGE_VAL;
+    maxValue = -HUGE_VAL;
 }
-- (NSSet *)filterNames
+
+- (NSSet *)filterValues
 {
-    return filterNames;
+    return filterValues;
 }
 
 
@@ -225,8 +213,8 @@
                                       minDuration:(double)minDuration
 {
     NSEnumerator *origEnum;
-    NSNumber *min;
-    NSNumber *max;
+    double min;
+    double max;
     BOOL limitsChanged = NO;
 
     if (array != nil
@@ -249,23 +237,77 @@
                                         startTime:start
                                           endTime:end
                                        enumerator:origEnum
-                                       nameFilter:filterNames];
+                                      valueFilter:filterValues];
 
     [entityClass getMinValue:&min
                     maxValue:&max
                     forArray:array
                pajeComponent:component];
-    if ((minValue == nil) || ([minValue compare:min] == NSOrderedDescending)) {
-        Assign(minValue, min);
+    if (min < minValue) {
+        minValue = min;
         limitsChanged = YES;
     }
-    if ((maxValue == nil) || ([maxValue compare:max] == NSOrderedAscending)) {
-        Assign(maxValue, max);
+    if (max > maxValue) {
+        maxValue = max;
         limitsChanged = YES;
     }
-    if (limitsChanged)
-        [component dataChangedForEntityType:self];
+    if (limitsChanged) {
+        [component limitsChangedForEntityType:self];
+    }
 
     return [array reverseObjectEnumeratorOfClass:entityClass];
 }
+
+- (NSEnumerator *)enumeratorOfCompleteEntitiesInContainer:(PajeContainer *)container
+                                                 fromTime:(NSDate *)start
+                                                   toTime:(NSDate *)end
+                                              minDuration:(double)minDuration
+{
+    NSEnumerator *origEnum;
+    double min;
+    double max;
+    BOOL limitsChanged = NO;
+
+    if (array != nil
+        && [container isEqual:[array container]]
+        && ![start isEarlierThanDate:[array startTime]]
+        && ![end isLaterThanDate:[array endTime]]) {
+        return [array completeObjectEnumeratorOfClass:entityClass
+                                             fromTime:start
+                                               toTime:end];
+    }
+
+    if (array != nil) [array release];
+    origEnum = [component enumeratorOfEntitiesTyped:entityTypeToReduce
+                                        inContainer:container
+                                           fromTime:start
+                                             toTime:end
+                                        minDuration:0];
+    array = [[BusyArray alloc] initWithEntityType:self
+                                        container:container
+                                        startTime:start
+                                          endTime:end
+                                       enumerator:origEnum
+                                      valueFilter:filterValues];
+
+    [entityClass getMinValue:&min
+                    maxValue:&max
+                    forArray:array
+               pajeComponent:component];
+    if (min < minValue) {
+        minValue = min;
+        limitsChanged = YES;
+    }
+    if (max > maxValue) {
+        maxValue = max;
+        limitsChanged = YES;
+    }
+    if (limitsChanged) {
+        [component limitsChangedForEntityType:self];
+    }
+
+    return [array completeObjectEnumeratorOfClass:entityClass
+                                         fromTime:start
+                                           toTime:end];
+}
 @end
