@@ -22,10 +22,24 @@
 // methods for finding entities in DrawView.
 
 #include "DrawView.h"
-#ifdef GNUSTEP
+
 #include <math.h>
-#include <values.h>
-#endif
+
+
+// returns a rectangle that is the same as rect but has positive height and width
+static NSRect positiveRect(NSRect rect)
+{
+    if (rect.size.height < 0) {
+        rect.origin.y += rect.size.height;
+        rect.size.height *= -1;
+    }
+    if (rect.size.width < 0) {
+        rect.origin.x += rect.size.width;
+        rect.size.width *= -1;
+    }
+    return rect;
+}
+
 
 @implementation DrawView (Finding)
 /*
@@ -70,10 +84,12 @@ BOOL line_hit(double px, double py,
     NSRect r;
     int hit;
 #ifndef GNUSTEP
+#ifndef __APPLE__
     shapefunction *path;
     STEntityTypeLayout *entityDescriptor;
 #endif
-
+#endif
+if (![entity isKindOfClass:[PajeEntity class]]) return YES;
     // couldn't make PSinfill work with an arrow...
     switch ([filter drawingTypeForEntity:entity]) {
         case PajeLinkDrawingType:
@@ -88,7 +104,7 @@ BOOL line_hit(double px, double py,
             break;
         default:
             r = [self rectForEntity:entity];
-#ifdef GNUSTEP
+#if defined(GNUSTEP) || defined(__APPLE__) // REVER ISSO
             hit = NSPointInRect(p, r);
 #else
             // test insideness with x=y=0, because PostScript has a problem with
@@ -134,12 +150,12 @@ BOOL line_hit(double px, double py,
     PajeEntity *closestEntity = nil;
     PajeEntity *entity;
 
-    if ([layoutDescriptor drawingType] != PajeLinkDrawingType) {
-    NSRect rect = [layoutDescriptor rectInContainer:container];
-
-    if (!NSPointInRect(point, rect)) {
-        return nil;
+    if ([layoutDescriptor drawingType] == PajeVariableDrawingType) {
+//        return nil;
     }
+
+    if (![layoutDescriptor isPoint:point inContainer:container]) {
+        return nil;
     }
 
     float width;
@@ -239,13 +255,13 @@ BOOL line_hit(double px, double py,
     PajeEntity *entity;
     NSMutableArray *array;
 
-    if ([layoutDescriptor drawingType] != PajeLinkDrawingType) {
-    NSRect rect = [layoutDescriptor rectInContainer:container];
+//    if ([layoutDescriptor drawingType] == PajeVariableDrawingType) {
+//        return nil;
+//    }
 
-    if (!NSPointInRect(point, rect)) {
+    if (![layoutDescriptor isPoint:point inContainer:container]) {
         return nil;
     }
-    }
 
     array = [NSMutableArray array];
     
@@ -370,8 +386,8 @@ BOOL line_hit(double px, double py,
         case PajeVariableDrawingType:
             x1 = TIMEtoX([filter startTimeForEntity:entity]);
             x2 = TIMEtoX([filter endTimeForEntity:entity]);
-            min = [[filter minValueForEntityType:entityType] floatValue];
-            max = [[filter maxValueForEntityType:entityType] floatValue];
+            min = [filter minValueForEntityType:entityType];
+            max = [filter maxValueForEntityType:entityType];
 
             if (min != max) {
                 scale = -([layoutDescriptor height] - 4) / (max - min);
@@ -379,7 +395,7 @@ BOOL line_hit(double px, double py,
                 scale = 1;
             }
             offset = [layoutDescriptor yInContainer:[filter containerForEntity:entity]] + 2 - max * scale;
-            y1 = [[filter valueForEntity:entity] doubleValue] * scale + offset;
+            y1 = max/*[filter doubleValueForEntity:entity]*/ * scale + offset;
             y2 = min * scale + offset;
             rect.origin.x = x1;
             rect.size.width = (x2 - x1);
@@ -411,4 +427,20 @@ BOOL line_hit(double px, double py,
 
     return rect;
 }
+
+- (NSRect)drawRectForEntity:(PajeEntity *)entity
+{
+    NSRect rect;
+    rect = positiveRect([self rectForEntity:entity]);
+    
+    return rect;
+}
+
+- (NSRect)highlightRectForEntity:(PajeEntity *)entity
+{
+    NSRect rect;
+    rect = positiveRect([self rectForEntity:entity]);
+    rect = NSInsetRect(rect, -13, -13); // should be 10+lineWidth
+    return rect;
+}
 @end
