@@ -22,143 +22,88 @@
 
 #define SHOW_TYPES
 
+
+#define CEIL(f) (1000-((int)(1000-(f))))
+#define CEIL2(f) (1000-((int)(999.5-(f))))
+
 @interface VCell : NSCell
 @end
 @implementation VCell
 - (void) _drawAttributedText: (NSAttributedString*)aString 
 		     inFrame: (NSRect)aRect
 {
-  NSSize titleSize;
-  NSRect newRect;
+    NSSize titleSize;
+    NSRect newRect;
 
-  if (aString == nil)
-    return;
-
-/*
-[aString drawInRect:aRect];
-return;
-
-  if (NSWidth(aRect) > NSHeight(aRect)) {
-    [super _drawAttributedText: aString inFrame:aRect];
-    return;
-  }
-//*/
-
-  titleSize = [aString size];
-
-  /** Important: text should always be vertically centered without
-   * considering descender [as if descender did not exist].
-   * This is particularly important for single line texts.
-   * Please make sure the output remains always correct.
-   */
-//  aRect.origin.y = NSMidY (aRect) - titleSize.height/2; 
-//  aRect.size.height = titleSize.height;
-  if (NSWidth(aRect) > NSHeight(aRect)) {
-
-  if (titleSize.width < aRect.size.width) {
-    newRect.origin.x = NSMidX(aRect) - titleSize.width/2;
-    newRect.origin.y = NSMidY(aRect) - titleSize.height/2;
-    newRect.size = titleSize;
-    //newRect.size.width = aRect.size.height;
-  } else {
-    newRect.origin.x = NSMinX(aRect);
-    newRect.size.width = aRect.size.width + 1;
-    newRect.size.height = titleSize.height 
-                        * (1000-(int)(1000-(titleSize.width / newRect.size.width)));
-    if (newRect.size.height > aRect.size.height) {
-        newRect.size.height = aRect.size.height;
+    if (aString == nil || NSWidth(aRect) < 5 || NSHeight(aRect) < 5) {
+        return;
     }
-    newRect.origin.y = NSMidY(aRect) - newRect.size.height/2;
-  }
-  //PSgsave();
-  //PStranslate(newRect.origin.x, newRect.origin.y);
-  //newRect.origin = NSMakePoint(0, 0);
-  //PSrotate(-90);
-  [aString drawInRect: newRect];
-  //PSgrestore();
-  
-  } else {
-  
-  if (titleSize.width < aRect.size.height) {
-    newRect.origin.x = NSMidX(aRect) - titleSize.height/2;
-    newRect.origin.y = NSMidY(aRect) + titleSize.width/2;
-    newRect.size = titleSize;
-    newRect.size.width += 6;
-  } else {
-    newRect.origin.y = NSMaxY(aRect);
-    newRect.size.width = aRect.size.height + 1;
-    newRect.size.height = titleSize.height 
-                        * (1000-(int)(999.5-(titleSize.width / newRect.size.width)));
-    if (newRect.size.height > aRect.size.width) {
-        newRect.size.height = aRect.size.width;
-    }
-    newRect.origin.x = NSMidX(aRect) - newRect.size.height/2;
-  }
-  PSgsave();
-  PStranslate(newRect.origin.x, newRect.origin.y);
-  newRect.origin = NSMakePoint(0, 0);
-  PSrotate(-90);
-  [aString drawInRect: newRect];
-  PSgrestore();
 
-  }
+    titleSize = [aString size];
+
+#if defined __MINGW32__
+    if (YES) {
+#else
+    if (NSWidth(aRect) > NSHeight(aRect)) {
+#endif
+
+        if (titleSize.width < aRect.size.width) {
+            newRect.origin.x = NSMidX(aRect) - titleSize.width/2;
+            newRect.origin.y = NSMidY(aRect) - titleSize.height/2;
+            newRect.size = titleSize;
+            //newRect.size.width = aRect.size.height;
+        } else {
+            newRect.origin.x = NSMinX(aRect);
+            newRect.size.width = aRect.size.width + 1;
+            newRect.size.height = titleSize.height 
+                                * CEIL(titleSize.width / newRect.size.width);
+            if (newRect.size.height > aRect.size.height) {
+                newRect.size.height = aRect.size.height;
+            }
+            newRect.origin.y = NSMidY(aRect) - newRect.size.height/2;
+        }
+        [aString drawInRect: newRect];
+  
+    } else {
+  
+        if (titleSize.width < aRect.size.height) {
+            newRect.origin.x = NSMidX(aRect) - titleSize.height/2;
+            newRect.origin.y = NSMidY(aRect) + titleSize.width/2;
+            newRect.size = titleSize;
+            newRect.size.width += 6;
+        } else {
+            newRect.origin.y = NSMaxY(aRect);
+            newRect.size.width = aRect.size.height + 1;
+            newRect.size.height = titleSize.height 
+                                * CEIL2(titleSize.width / newRect.size.width);
+            if (newRect.size.height > aRect.size.width) {
+                newRect.size.height = aRect.size.width;
+            }
+            newRect.origin.x = NSMidX(aRect) - newRect.size.height/2;
+        }
+        NSAffineTransform *transf;
+        transf = [NSAffineTransform transform];
+        [transf translateXBy: newRect.origin.x yBy: newRect.origin.y];
+        [transf rotateByDegrees:-90];
+        [transf concat];
+        newRect.origin = NSMakePoint(0, 0);
+        [aString drawInRect: newRect];
+        [transf invert];
+        [transf concat];
+
+    }
 }
+
 - (void) drawInteriorWithFrame: (NSRect)cellFrame inView: (NSView*)controlView
 {
-  if (![controlView window])
-    return;
-
-  cellFrame = [self drawingRectForBounds: cellFrame];
-
-  //FIXME: Check if this is also neccessary for images,
-  // Add spacing between border and inside 
-  if (_cell.is_bordered || _cell.is_bezeled)
-    {
-      cellFrame.origin.x += 3;
-      cellFrame.size.width -= 6;
-      cellFrame.origin.y += 1;
-      cellFrame.size.height -= 2;
+    if ([controlView window] == nil) {
+        return;
     }
 
-  switch (_cell.type)
-    {
-      case NSTextCellType:
-        {
-	  [self _drawAttributedText: [self attributedStringValue]
-		inFrame: cellFrame];
-	}
-	break;
+    cellFrame = [self drawingRectForBounds: cellFrame];
 
-      case NSImageCellType:
-	if (_cell_image)
-	  {
-	    NSSize size;
-	    NSPoint position;
-
-	    size = [_cell_image size];
-	    position.x = MAX(NSMidX(cellFrame) - (size.width/2.),0.);
-	    position.y = MAX(NSMidY(cellFrame) - (size.height/2.),0.);
-	    /*
-	     * Images are always drawn with their bottom-left corner
-	     * at the origin so we must adjust the position to take
-	     * account of a flipped view.
-	     */
-	    if ([controlView isFlipped])
-	      position.y += size.height;
-	    [_cell_image compositeToPoint: position operation: NSCompositeSourceOver];
-	  }
-	 break;
-
-      case NSNullCellType:
-         break;
-    }
-
-  if (_cell.shows_first_responder)
-    NSDottedFrameRect(cellFrame);
-
-  // NB: We don't do any highlighting to make it easier for subclasses
-  // to reuse this code while doing their own custom highlighting and
-  // prettyfying
+    [self _drawAttributedText: [self attributedStringValue]
+                      inFrame: cellFrame];
 }
 @end
 
@@ -323,6 +268,134 @@ if (maxLevel == 0) maxLevel = 1;
     [[self enclosingScrollView] tile];
 }
 
+- (void)drawHashMarksAndLabelsInRect:(NSRect)drawRect
+                   forVariableLayout:(STVariableTypeLayout *)layout
+{
+    float vMin;
+    float vMax;
+    float yScale;
+    float yOffset;
+    PajeEntityType *entityType;
+    
+    entityType = [layout entityType];
+
+    vMin = [layout minValue];
+    vMax = [layout maxValue];
+    if (vMax <= vMin) {
+        vMin = [controller minValueForEntityType:entityType];
+        vMax = [controller maxValueForEntityType:entityType];
+    }
+
+    if (vMin != vMax) {
+        yScale = -([layout height] - 4) / (vMax - vMin);
+    } else {
+        yScale = 1;
+    }
+
+    yOffset = NSMinY(drawRect) + 2 - (vMax * yScale);
+
+    //y = [filter valueForEntity:entity] * yScale + yOffset;
+    [[NSColor controlColor] set];
+    //NSRectFill(drawRect);
+    NSBezierPath *path;
+    path = [NSBezierPath bezierPath];
+    [path moveToPoint:NSMakePoint(NSMaxX(drawRect), NSMinY(drawRect) + 2)];
+    [path lineToPoint:NSMakePoint(NSMaxX(drawRect), NSMaxY(drawRect) - 2)];
+    
+    NSFont *font = [NSFont systemFontOfSize: [NSFont smallSystemFontSize]];
+    NSDictionary *attr = [[NSDictionary alloc] initWithObjectsAndKeys:
+                               font, NSFontAttributeName,
+                               [NSColor blackColor], NSForegroundColorAttributeName,
+                               nil];
+
+    NSEnumerator *en = [[layout hashMarkValues] objectEnumerator];
+    NSNumber *n;
+    while ((n = [en nextObject]) != nil) {
+        NSSize size;
+        NSString *str;
+        NSString *hashValueFormat = [layout hashValueFormat];
+        float v = [n floatValue];
+        float y = v * yScale + yOffset;
+        float x;
+        [path moveToPoint:NSMakePoint(NSMaxX(drawRect) - 4, y)];
+        [path lineToPoint:NSMakePoint(NSMaxX(drawRect), y)];
+        str = [NSString stringWithFormat:hashValueFormat, v];
+        size = [str sizeWithAttributes:attr];
+        x = NSMaxX(drawRect) - size.width;
+        if (y > NSMinY(drawRect) + size.height + 1) {
+            [str drawAtPoint:NSMakePoint(x, y - size.height + 2)
+              withAttributes:attr];
+        } else {
+            [str drawAtPoint:NSMakePoint(x, y + 1)
+              withAttributes:attr];
+        }
+    }
+    [[NSColor blackColor] set];
+    [path stroke];
+    [attr release];
+}
+- (void)drawHashMarksAndLabelsInRect:(NSRect)drawRect
+                  forContainerLayout:(STContainerTypeLayout *)layout
+{
+    float vMin;
+    float vMax;
+    float yScale;
+    float yOffset;
+    PajeEntityType *entityType;
+    
+    entityType = [layout entityType];
+
+    vMin = [layout minValue];
+    vMax = [layout maxValue];
+    if (vMax <= vMin) {
+        return;
+    }
+
+    yScale = -([layout heightForVariables] - 4) / (vMax - vMin);
+
+    yOffset = NSMinY(drawRect) + 2 - (vMax * yScale)
++ [layout variablesOffset] ;
+    //y = [filter valueForEntity:entity] * yScale + yOffset;
+    [[NSColor controlColor] set];
+    //NSRectFill(drawRect);
+    NSBezierPath *path;
+    path = [NSBezierPath bezierPath];
+    [path moveToPoint:NSMakePoint(NSMaxX(drawRect), NSMinY(drawRect) + 2)];
+    [path lineToPoint:NSMakePoint(NSMaxX(drawRect), NSMaxY(drawRect) - 2)];
+    
+    NSFont *font = [NSFont systemFontOfSize: [NSFont smallSystemFontSize]];
+    NSDictionary *attr = [[NSDictionary alloc] initWithObjectsAndKeys:
+                               font, NSFontAttributeName,
+                               [NSColor blackColor], NSForegroundColorAttributeName,
+                               nil];
+
+    NSEnumerator *en = [[layout hashMarkValues] objectEnumerator];
+    NSNumber *n;
+    while ((n = [en nextObject]) != nil) {
+        NSSize size;
+        NSString *str;
+        NSString *hashValueFormat = [layout hashValueFormat];
+        float v = [n floatValue];
+        float y = v * yScale + yOffset;
+        float x;
+        [path moveToPoint:NSMakePoint(NSMaxX(drawRect) - 4, y)];
+        [path lineToPoint:NSMakePoint(NSMaxX(drawRect), y)];
+        str = [NSString stringWithFormat:hashValueFormat, v];
+        size = [str sizeWithAttributes:attr];
+        x = NSMaxX(drawRect) - size.width;
+        if (y > NSMinY(drawRect) + size.height + 1) {
+            [str drawAtPoint:NSMakePoint(x, y - size.height + 2)
+              withAttributes:attr];
+        } else {
+            [str drawAtPoint:NSMakePoint(x, y + 1)
+              withAttributes:attr];
+        }
+    }
+    [[NSColor blackColor] set];
+    [path stroke];
+    [attr release];
+}
+
 - (void)drawHashMarksAndLabelsInRect:(NSRect)drawRect
 {
 int x;
@@ -347,6 +420,13 @@ if (x != maxLevel/*[thicknesses count]*/) {
     NSEnumerator *sublayoutEnum;
     STContainerTypeLayout *sublayout;
 
+if ([layout heightForVariables] > 10) {
+NSRect r = [layout rectOfInstance:container];
+r.origin.x = NSMinX(drawRect);
+r.origin.y -= offset;
+r.size.width = NSMaxX([self bounds]) - r.origin.x - 1;
+[self drawHashMarksAndLabelsInRect:r forContainerLayout:layout];
+}
     sublayoutEnum = [[layout subtypes] objectEnumerator];
     while ((sublayout = [sublayoutEnum nextObject]) != nil) {
         [self drawContainerLayout:sublayout
@@ -396,6 +476,8 @@ if (x != maxLevel/*[thicknesses count]*/) {
         }
         if (r.size.width > 2 && [layout drawingType] == PajeVariableDrawingType) {
             [self widthForLevel:level];
+//r.size.width = NSMaxX([self bounds]) - r.origin.x - 1;
+//[self drawHashMarksAndLabelsInRect:r forVariableLayout:layout];
         }
         if (![layout isContainer]) return;
         level++;
@@ -422,7 +504,7 @@ if (x != maxLevel/*[thicknesses count]*/) {
                         [[NSColor whiteColor] set];
                         NSRectFill(r);
                     }
-                    [cell setStringValue:[instance description]];
+                    [cell setStringValue:[instance name/*description*/]];
                     [cell drawWithFrame:NSInsetRect(r, -.5, -.5) inView:self];
                 }
             }
@@ -785,7 +867,8 @@ if (x != maxLevel/*[thicknesses count]*/) {
         while ((orgContainer = [contEnum nextObject]) != nil) {
             if ([orgContainer isEqual:containerBeingDragged]) {
                 continue;
-            } else if ([orgContainer isEqual:droppedInstance]) {                                 if (!before) [order addObject:orgContainer];
+            } else if ([orgContainer isEqual:droppedInstance]) {
+                if (!before) [order addObject:orgContainer];
                 [order addObject:containerBeingDragged];
                 if (before) [order addObject:orgContainer];
             } else {
