@@ -24,6 +24,7 @@
 #include "DrawView.h"
 #include "STEntityTypeLayoutController.h"
 #include "../Paje/PajeTraceController.h"
+#include "../General/Macros.h"
 
 @implementation STController
 
@@ -34,10 +35,20 @@
     if (self != nil) {
         if (![NSBundle loadNibNamed:@"SpaceTime" owner:self]) {
             NSRunAlertPanel(@"SpaceTime", @"Couldn't load interface file",
-                            nil, nil, nil);
-        }
+                            @"Abort", nil, nil);
+            //[self release];
+            //return nil;
+        //} else {
+    }    [self adjustInterface];
+        //}
 
         layoutDescriptors = [[NSMutableDictionary alloc] init];
+        layoutController = [[STEntityTypeLayoutController alloc]
+                                            initWithDelegate:self];
+
+        // register the tools with the controller
+        [self registerTool:self];
+        [self registerTool:layoutController];
     }
 
     return self;
@@ -45,8 +56,7 @@
 
 - (void)dealloc
 {
-    [layoutDescriptors release];
-    [layoutController release];
+    // super's implementation will grant disconnection
 #ifdef GNUSTEP
     // workaround GNUstep bug #13382
     {
@@ -64,7 +74,15 @@
     [super dealloc];
 }
 
-- (void)awakeFromNib
+- (void)disconnectComponent
+{
+    Assign(layoutDescriptors, nil);
+    Assign(layoutController, nil);
+    [super disconnectComponent];
+}
+
+// set some things that weren't set when main nib file was loaded
+- (void)adjustInterface
 {
     // set rulers
     [scrollView setHasVerticalRuler:YES];
@@ -73,7 +91,7 @@
     [scrollView setVerticalRulerView:hierarchyRuler];
     [hierarchyRuler setClientView:drawView];
 
-    [scrollView setHasHorizontalRuler:/*NO];*/YES];
+    [scrollView setHasHorizontalRuler:YES];
     [[scrollView horizontalRulerView] setClientView:drawView];
     [[scrollView horizontalRulerView] setReservedThicknessForMarkers:0.0];
 
@@ -82,12 +100,6 @@
     [window setDelegate:self];
     [window setFrameAutosaveName:@"SpaceTime"];
     [window makeKeyAndOrderFront:self];
-
-    // register the tools with the controller
-    [self registerTool:self];
-    layoutController = [[STEntityTypeLayoutController alloc]
-                                          initWithDelegate:self];
-    [self registerTool:layoutController];
 }
 
 - (void)windowDidBecomeKey:(NSNotification *)notification
@@ -112,10 +124,10 @@
     return @"Space Time Diagram";
 }
 
-- (void)setInputComponent:(PajeComponent *)filter
+- (void)setInputComponent:(PajeFilter *)filter
 {
     [super setInputComponent:filter];
-    [drawView setFilter:(PajeFilter *)filter];
+    [drawView setFilter:self/*(PajeFilter *)filter*/];
 }
 
 
@@ -139,6 +151,10 @@
                                                  drawingType:drawingType
                                          containerDescriptor:cDesc
                                                   controller:self];
+    if (drawingType == PajeVariableDrawingType) {
+        [(STVariableTypeLayout*)layoutDescriptor setMinValue:[self minValueForEntityType:eType]];
+        [(STVariableTypeLayout*)layoutDescriptor setMaxValue:[self maxValueForEntityType:eType]];
+    }
     [layoutDescriptors setObject:layoutDescriptor forKey:eType];
 
     subtypeEnum = [[self containedTypesForContainerType:eType]
@@ -189,10 +205,14 @@
 }
 
 
+- (void)timeLimitsChanged
+{
+    [self hierarchyChanged];
+}
 
 - (void)hierarchyChanged
 {
-    [window setTitleWithRepresentedFilename:[self nameForEntity:[self rootInstance]]];
+    [window setTitleWithRepresentedFilename:[self nameForContainer:[self rootInstance]]];
 
     if ([self startTime] == nil) return;
 
@@ -223,6 +243,21 @@
 //[scrollView setNeedsDisplay:YES];
 }
 
+- (void)limitsChangedForEntityType:(PajeEntityType *)entityType
+{
+    STEntityTypeLayout *layoutDescriptor;
+    layoutDescriptor = [layoutDescriptors objectForKey:entityType];
+    if (layoutDescriptor != nil
+        && [layoutDescriptor drawingType] == PajeVariableDrawingType) {
+        [(STVariableTypeLayout*)layoutDescriptor
+                        setMinValue:[self minValueForEntityType:entityType]];
+        [(STVariableTypeLayout*)layoutDescriptor
+                        setMaxValue:[self maxValueForEntityType:entityType]];
+    }
+    // FIXME: redrawing could be limeted to rects of this entityType.
+    [drawView setNeedsDisplay:YES];
+}
+
 - (void)changedTimeScale
 {
     [self calcRectOfInstance:[self rootInstance]
@@ -265,4 +300,16 @@
 //    [[NSPrintOperation printOperationWithView:[window contentView]] runOperation]; 
     [window print:sender];
 }
+
+- (BOOL)isSelectedEntity:(id)entity
+{
+    id highlightedEntities;
+    highlightedEntities = [drawView highlightedEntities];
+    if (highlightedEntities != nil) {
+        if ([highlightedEntities containsObject:entity]) {
+            return YES;
+        }
+    }
+    return [super isSelectedEntity:entity];
+}
 @end
