@@ -325,6 +325,8 @@
     Assign(labelLinesPath, nil);
     Assign(labelAttributes, nil);
     Assign(complementAttributes, nil);
+
+    [super dealloc];
 }
 
 // The data provider retains the cells, so they cannot retain it.
@@ -403,6 +405,7 @@
 
 - (void)drawInteriorWithFrameVBar:(NSRect)cellFrame inView:(NSView *)controlView
 {
+#ifdef VBARSUPPORT
     float x, y, h, hmax, w, wmax;
     float val, total, max, min, cnt;
     float fontHeight = 0;
@@ -481,7 +484,7 @@
     unsigned index;
     unsigned count = [data subCount];
     for (index = 0; index < count && index < 10; index++) {
-        val = [data subValueAtIndex:index];
+        val = [data subDoubleValueAtIndex:index];
 
         // draw bar
         [[data subColorAtIndex:index] set];
@@ -506,7 +509,7 @@
             }
             [attributes setObject:[NSColor blackColor]
                            forKey:NSForegroundColorAttributeName];
-            [[data subNameAtIndex:index] drawAtRTPoint:NSMakePoint(0, 0)
+            [[data subValueAtIndex:index] drawAtRTPoint:NSMakePoint(0, 0)
                          withAttributes:attributes];
             PSgrestore();
             
@@ -547,6 +550,7 @@
 
         x += w;
     }
+#endif
 }
 
 - (void)addPieSliceWithName:(NSString *)sliceName
@@ -730,14 +734,14 @@
     angle = angle0;
     count = [data subCount];
     for (index = 0; index < count; index++) {
-        val = [data subValueAtIndex:index];
+        val = [data subDoubleValueAtIndex:index];
         if ((val/total) < .05 && (nothers > 0 || index < (count-1))) {
             others += val;
             nothers++;
             continue;
         }
 
-        [self addPieSliceWithName:[data subNameAtIndex:index]
+        [self addPieSliceWithName:[data subValueAtIndex:index]
                      initialAngle:angle
                             color:[data subColorAtIndex:index]
                             value:val
