@@ -32,8 +32,6 @@
 #include "../General/Macros.h"
 #include "../General/NSDate+Additions.h"
 
-#include "StateAggregator.h"
-
 @implementation Encapsulate
 
 - (id)initWithController:(PajeTraceController *)c
@@ -69,13 +67,8 @@
     }
 }
 
-- (void)inputEntity:(id)entity
-/*"There's new data in the input port. Encapsulate it."*/
+- (void)activateTimer
 {
-    //[self addEntity:entity];
-    [self addChunk:entity];
-    traceChanged = YES;
-
     if (!timerActive) {
         [self performSelector:@selector(timeElapsed:)
                    withObject:self
@@ -84,12 +77,30 @@
     }
 }
 
+- (void)hierarchyChanged
+{
+    hierarchyChanged = YES;
+    [self activateTimer];
+}
+
+- (void)timeLimitsChanged
+{
+    timeLimitsChanged = YES;
+    [self activateTimer];
+}
+
+- (void)inputEntity:(id)entity
+/*"There's new data in the input port. Encapsulate it."*/
+{
+    //[self addEntity:entity];
+    [self addChunk:entity];
+    traceChanged = YES;
+
+    [self activateTimer];
+}
+
 - (void)addChunk:(EntityChunk *)chunk
 {
-    PajeEntityType *entityType = [chunk entityType];
-    PajeContainer *container = [chunk container];
-    ChunkArray *chunks;
-    
     //KLUDGE to identify last date in memory.
     // (doesn't work if events are not seen)
     // FIXME this should be calculated on demand
@@ -103,27 +114,9 @@
         Assign(startTimeInMemory, time);
     }
     //ENDKLUDGE
-
-    NSMutableDictionary *dict = [entityLists objectForKey:entityType];
-    if (dict == nil) {
-        // unknown entity type -- create a new dict for it
-        dict = [[NSMutableDictionary alloc] init];
-        [entityLists setObject:dict forKey:entityType];
-        [dict release];
-    }
-
-    chunks = [dict objectForKey:container];
-
-    if (chunks == nil) {
-        // unknown container for this entity type -- create a new array for it
-        chunks = [[ChunkArray alloc] init];
-        [dict setObject:chunks forKey:container];
-        [chunks release]; // dict is retaining it
-    }
-
-    [chunks addChunk:chunk];
 }
 
+
 - (void)send
 /*"notifies that data has changed."*/
 {
@@ -131,11 +124,17 @@
     // (or else space time diagram will think trace has no data, and the
     // lazy reading will not work well)
     // FIXME should send more specific notification
-    [self hierarchyChanged];
-    traceChanged = NO;
+    if (hierarchyChanged) {
+        [super hierarchyChanged];
+        hierarchyChanged = NO;
+    }
+    if (timeLimitsChanged) {
+        [super timeLimitsChanged];
+        timeLimitsChanged = NO;
+    }
     [self performSelector:@selector(timeElapsed:)
                withObject:self
-               afterDelay:0.5];
+               afterDelay:0.005];
     timerActive = YES;
 }
 
@@ -176,6 +175,7 @@
 
 - (void)verifyStartTime:(NSDate *)start endTime:(NSDate *)end
 {
+return;
     NSDebugMLLog(@"tim", @"[%@:%@] [%@:%@]",
                 start, end, startTimeInMemory, endTimeInMemory);
     if ([start isEarlierThanDate:startTimeInMemory]
@@ -187,6 +187,8 @@
             nil];
         NSDebugMLLog(@"tim", @"notify [%@:%@] [%@:%@]",
                     start, end, startTimeInMemory, endTimeInMemory);
+        NSLog(@"fault:[%@:%@] [%@:%@]",
+                    start, end, startTimeInMemory, endTimeInMemory);
         [[NSNotificationCenter defaultCenter]
                       postNotificationName:@"PajeTraceNotInMemoryNotification"
                                     object:self
@@ -199,98 +201,100 @@
                                    fromTime:(NSDate *)start
                                      toTime:(NSDate *)end
 {
-    NSDictionary *dict;
-    ChunkArray *chunks;
-    NSEnumerator *subcontainerEnum;
-    id subcontainer;
-    MultiEnumerator *multiEnum;
-    PajeEntityType *parentType;
-    
-    [self verifyStartTime:start endTime:end];
-    
-    dict = [entityLists objectForKey:entityType];
-    // if container directly contains entityType's
-    chunks = [dict objectForKey:container];
-    if (chunks != nil) {
-        return [chunks enumeratorOfEntitiesFromTime:start toTime:end];
-    }
-
-    // if container should contain entityType directly, we have no entities!
-    if ([[entityType containerType] isEqual:[container entityType]]) {
-        return nil;
-    }
-    
-    // container does not contain entityType's directly.
-    // Try containers in-between.
-    parentType = [entityType containerType];
-    subcontainerEnum = [self enumeratorOfContainersTyped:parentType
-                                             inContainer:container];
-    multiEnum = [MultiEnumerator enumerator];
-    while ((subcontainer = [subcontainerEnum nextObject]) != nil) {
-        chunks = [dict objectForKey:subcontainer];
-        if (chunks != nil) {
-            [multiEnum addEnumerator:[chunks enumeratorOfEntitiesFromTime:start
-                                                                   toTime:end]];
-        }
-    }
-    return multiEnum;
+    return [container enumeratorOfEntitiesTyped:entityType
+                                       fromTime:start
+                                         toTime:end];
 }
 
+- (NSEnumerator *)enumeratorOfCompleteEntitiesTyped:(PajeEntityType *)entityType
+                                        inContainer:(PajeContainer *)container
+                                           fromTime:(NSDate *)start
+                                             toTime:(NSDate *)end
+{
+    return [container enumeratorOfCompleteEntitiesTyped:entityType
+                                               fromTime:start
+                                                 toTime:end];
+}
+
+
 - (NSEnumerator *)enumeratorOfEntitiesTyped:(PajeEntityType *)entityType
                                 inContainer:(PajeContainer *)container
                                    fromTime:(NSDate *)start
                                      toTime:(NSDate *)end
                                 minDuration:(double)minDuration
 {
-    NSEnumerator *en;
-    
-    if (0 && minDuration > 1e-6) {
-    en = [self enumeratorOfEntitiesTyped:entityType
-                             inContainer:container
-                                fromTime:[start addTimeInterval:-minDuration]
-                                  toTime:[end addTimeInterval:minDuration]
-                             minDuration:minDuration/2];
-    } else
-    en = [self enumeratorOfEntitiesTyped:entityType
-                             inContainer:container
-                                fromTime:[start addTimeInterval:-minDuration]
-                                  toTime:[end addTimeInterval:minDuration]];
-    if (minDuration <= 1e-6
-        || ([entityType drawingType] != PajeStateDrawingType
-            && [entityType drawingType] != PajeEventDrawingType)) {
-        return en;
-    }
-    EntityAggregator *aggregator;
-    PajeEntity *state;
-    NSMutableArray *filtered;
-    PajeEntity *aggregate;
-    
-    if ([entityType drawingType] == PajeStateDrawingType) {
-        aggregator = [StateAggregator aggregatorWithMaxDuration:minDuration];
-    } else {
-        aggregator = [EventAggregator aggregatorWithMaxDuration:minDuration];
-    }
-    filtered = [NSMutableArray array];
-    while ((state = [en nextObject]) != nil) {
-        if (![aggregator addEntity:state]) {
-            aggregate = [aggregator aggregate];
-            if (aggregate != nil) {
-                [filtered addObject:aggregate];
-                if (![aggregator addEntity:state]) {
-                    [filtered addObject:state];
-                }
-            } else {
-                [filtered addObject:state];
-            }
+    if (![[entityType containerType] isEqual:[container entityType]]) {
+    NSEnumerator *subcontainerEnum;
+    id subcontainer;
+    MultiEnumerator *multiEnum;
+    PajeEntityType *parentType;
+
+    parentType = [entityType containerType];
+    subcontainerEnum = [self enumeratorOfContainersTyped:parentType
+                                             inContainer:container];
+    multiEnum = [MultiEnumerator enumerator];
+    while ((subcontainer = [subcontainerEnum nextObject]) != nil) {
+        NSEnumerator *en;
+        en = [self enumeratorOfEntitiesTyped:entityType
+                                inContainer:subcontainer
+                                   fromTime:start
+                                     toTime:end
+                                minDuration:minDuration];
+        if (en != nil) {
+            [multiEnum addEnumerator:en];
         }
     }
-    aggregate = [aggregator aggregate];
-    if (aggregate != nil) {
-        [filtered addObject:aggregate];
+    return multiEnum;
     }
-    return [filtered objectEnumerator];
+
+        return [self enumeratorOfEntitiesTyped:entityType
+                                   inContainer:container
+                                      fromTime:start
+                                        toTime:end];
 }
 
+
+- (NSEnumerator *)enumeratorOfCompleteEntitiesTyped:(PajeEntityType *)entityType
+                                        inContainer:(PajeContainer *)container
+                                           fromTime:(NSDate *)start
+                                             toTime:(NSDate *)end
+                                        minDuration:(double)minDuration
+{
+    // if container should contain entityType directly, we have no entities!
+    if (![[entityType containerType] isEqual:[container entityType]]) {
+    // container does not contain entityType's directly.
+    // Try containers in-between.
+        NSEnumerator *subcontainerEnum;
+        id subcontainer;
+        MultiEnumerator *multiEnum;
+        PajeEntityType *parentType;
+
+        parentType = [entityType containerType];
+        subcontainerEnum = [self enumeratorOfContainersTyped:parentType
+                                                 inContainer:container];
+        multiEnum = [MultiEnumerator enumerator];
+        while ((subcontainer = [subcontainerEnum nextObject]) != nil) {
+            NSEnumerator *en;
+            en = [self enumeratorOfCompleteEntitiesTyped:entityType
+                                             inContainer:subcontainer
+                                                fromTime:start
+                                                  toTime:end
+                                             minDuration:minDuration];
+            if (en != nil) {
+                [multiEnum addEnumerator:en];
+            }
+        }
+        return multiEnum;
+    }
+
+        return [self enumeratorOfCompleteEntitiesTyped:entityType
+                                           inContainer:container
+                                              fromTime:start
+                                                toTime:end];
+
+}
+
+
 - (NSEnumerator *)enumeratorOfContainersTyped:(PajeEntityType *)entityType
                                   inContainer:(PajeContainer *)container
 {
@@ -307,61 +311,18 @@
             }
         }
     }
-//    return [[instancesInContainer sortedArrayUsingSelector:@selector(compare:)]
-//                  objectEnumerator];
+    //[instancesInContainer sortUsingSelector:@selector(compare:)];
     return [instancesInContainer objectEnumerator];
 }
 
 
-- (void)removeObjectsBeforeTime:(NSDate *)time
+- (void)startChunk:(int)chunkNumber
 {
-    NSEnumerator *e;
-    NSMutableDictionary *d;
-
-    if (![time isLaterThanDate:startTimeInMemory]) {
-        return;
-    }
-
-    e = [entityLists objectEnumerator];
-    while ((d = [e nextObject]) != nil) {
-        NSEnumerator *e2 = [d objectEnumerator];
-        ChunkArray *l;
-        while ((l = [e2 nextObject]) != nil) {
-            [l removeChunksBeforeTime:time];
-        }
-    }
-
-    Assign(startTimeInMemory, time);
-
-    if ([time isLaterThanDate:endTimeInMemory]) {
-        Assign(endTimeInMemory, time);
-    }
+    // the components following this should not be interested in this method
 }
 
-- (void)removeObjectsAfterTime:(NSDate *)time
+- (void)endOfChunkLast:(BOOL)last
 {
-    NSEnumerator *e;
-    NSMutableDictionary *d;
-
-    if (![time isEarlierThanDate:endTimeInMemory]) {
-        return;
-    }
-
-    e = [entityLists objectEnumerator];
-    while ((d = [e nextObject]) != nil) {
-        NSEnumerator *e2 = [d objectEnumerator];
-        ChunkArray *l;
-        while ((l = [e2 nextObject]) != nil) {
-            [l removeChunksAfterTime:time];
-        }
-    }
-
-    Assign(endTimeInMemory, time);
-
-    if ([time isEarlierThanDate:startTimeInMemory]) {
-        Assign(startTimeInMemory, time);
-    }
+    // the components following this should not be interested in this method
 }
-
-
 @end
